from __future__ import annotations


import logging
import re
import requests
import time

from ghapi.core import GhApi
from fastcore.net import HTTP404NotFoundError, HTTP403ForbiddenError
from typing import Callable, Iterator, Optional
from unidiff import PatchSet

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)

# https://docs.github.com/en/get-started/writing-on-github/working-with-advanced-formatting/using-keywords-in-issues-and-pull-requests
PR_KEYWORDS = {
    "close",
    "closes",
    "closed",
    "fix",
    "fixes",
    "fixed",
    "resolve",
    "resolves",
    "resolved",
}


class Repo:
    def __init__(self, owner: str, name: str, token: Optional[str] = None):
        """
        Init to retrieve target repository and create ghapi tool

        Args:
            owner (str): owner of target repository
            name (str): name of target repository
            token (str): github token
        """
        self.owner = owner
        self.name = name
        self.token = token
        self.api = GhApi(token=token)
        self.repo = self.call_api(self.api.repos.get, owner=owner, repo=name)

    def call_api(self, func: Callable, **kwargs) -> dict | None:
        """
        API call wrapper with rate limit handling (checks every 5 minutes if rate limit is reset)

        Args:
            func (callable): API function to call
            **kwargs: keyword arguments to pass to API function
        Return:
            values (dict): response object of `func`
        """
        while True:
            try:
                values = func(**kwargs)
                return values
            except HTTP403ForbiddenError:
                while True:
                    rl = self.api.rate_limit.get()
                    logger.info(
                        f"[{self.owner}/{self.name}] Rate limit exceeded for token {self.token[:10]}, "
                        f"waiting for 5 minutes, remaining calls: {rl.resources.core.remaining}"
                    )
                    if rl.resources.core.remaining > 0:
                        break
                    time.sleep(60 * 5)
            except HTTP404NotFoundError:
                logger.info(f"[{self.owner}/{self.name}] Resource not found {kwargs}")
                return None

    def extract_resolved_issues(self, pull: dict) -> list[str]:
        """
        Extract list of issues referenced by a PR

        Args:
            pull (dict): PR dictionary object from GitHub
        Return:
            resolved_issues (list): list of issue numbers referenced by PR
        """
        # Define 1. issue number regex pattern 2. comment regex pattern 3. keywords
        issues_pat = re.compile(r"(\w+)\s+\#(\d+)")
        comments_pat = re.compile(r"(?s)<!--.*?-->")

        # Construct text to search over for issue numbers from PR body and commit messages
        text = pull.title if pull.title else ""
        text += "\n" + (pull.body if pull.body else "")
        commits = self.get_all_loop(
            self.api.pulls.list_commits, pull_number=pull.number, quiet=True
        )
        commit_messages = [commit.commit.message for commit in commits]
        commit_text = "\n".join(commit_messages) if commit_messages else ""
        text += "\n" + commit_text
        # Remove comments from text
        text = comments_pat.sub("", text)
        # Look for issue numbers in text via scraping <keyword, number> patterns
        references = issues_pat.findall(text)
        resolved_issues_set = set()
        if references:
            for word, issue_num in references:
                if word.lower() in PR_KEYWORDS:
                    resolved_issues_set.add(issue_num)
        return list(resolved_issues_set)

    def get_all_loop(
        self,
        func: Callable,
        per_page: int = 100,
        num_pages: Optional[int] = None,
        quiet: bool = False,
        **kwargs,
    ) -> Iterator:
        """
        Return all values from a paginated API endpoint.

        Args:
            func (callable): API function to call
            per_page (int): number of values to return per page
            num_pages (int): number of pages to return
            quiet (bool): whether to print progress
            **kwargs: keyword arguments to pass to API function
        """
        page = 1
        args = {
            "owner": self.owner,
            "repo": self.name,
            "per_page": per_page,
            **kwargs,
        }
        while True:
            try:
                # Get values from API call
                values = func(**args, page=page)
                yield from values
                if len(values) == 0:
                    break
                if not quiet:
                    rl = self.api.rate_limit.get()
                    logger.info(
                        f"[{self.owner}/{self.name}] Processed page {page} ({per_page} values per page). "
                        f"Remaining calls: {rl.resources.core.remaining}"
                    )
                if num_pages is not None and page >= num_pages:
                    break
                page += 1
            except Exception as e:
                # Rate limit handling
                logger.error(
                    f"[{self.owner}/{self.name}] Error processing page {page} "
                    f"w/ token {self.token[:10]} - {e}"
                )
                while True:
                    rl = self.api.rate_limit.get()
                    if rl.resources.core.remaining > 0:
                        break
                    logger.info(
                        f"[{self.owner}/{self.name}] Waiting for rate limit reset "
                        f"for token {self.token[:10]}, checking again in 5 minutes"
                    )
                    time.sleep(60 * 5)
        if not quiet:
            logger.info(
                f"[{self.owner}/{self.name}] Processed {(page - 1) * per_page + len(values)} values"
            )

    def get_all_issues(
        self,
        per_page: int = 100,
        num_pages: Optional[int] = None,
        direction: str = "desc",
        sort: str = "created",
        state: str = "closed",
        quiet: bool = False,
    ) -> Iterator:
        """
        Wrapper for API call to get all issues from repo

        Args:
            per_page (int): number of issues to return per page
            num_pages (int): number of pages to return
            direction (str): direction to sort issues
            sort (str): field to sort issues by
            state (str): state of issues to look for
            quiet (bool): whether to print progress
        """
        issues = self.get_all_loop(
            self.api.issues.list_for_repo,
            num_pages=num_pages,
            per_page=per_page,
            direction=direction,
            sort=sort,
            state=state,
            quiet=quiet,
        )
        return issues

    def get_all_pulls(
        self,
        per_page: int = 100,
        num_pages: Optional[int] = None,
        direction: str = "desc",
        sort: str = "created",
        state: str = "closed",
        quiet: bool = False,
    ) -> Iterator:
        """
        Wrapper for API call to get all PRs from repo

        Args:
            per_page (int): number of PRs to return per page
            num_pages (int): number of pages to return
            direction (str): direction to sort PRs
            sort (str): field to sort PRs by
            state (str): state of PRs to look for
            quiet (bool): whether to print progress
        """
        pulls = self.get_all_loop(
            self.api.pulls.list,
            num_pages=num_pages,
            direction=direction,
            per_page=per_page,
            sort=sort,
            state=state,
            quiet=quiet,
        )
        return pulls


def extract_problem_statement_and_hints(pull: dict, repo: Repo) -> tuple[str, str]:
    """
    Extract problem statement from issues associated with a pull request

    Args:
        pull (dict): PR dictionary object from GitHub
        repo (Repo): Repo object
    Return:
        text (str): problem statement
        hints (str): hints
    """
    text = ""
    all_hint_texts = list()
    for issue_number in pull["resolved_issues"]:
        issue = repo.call_api(
            repo.api.issues.get,
            owner=repo.owner,
            repo=repo.name,
            issue_number=issue_number,
        )
        if issue is None:
            continue
        title = issue.title if issue.title else ""
        body = issue.body if issue.body else ""
        text += f"{title}\n{body}\n"
        issue_number = issue.number
        hint_texts = _extract_hints(pull, repo, issue_number)
        hint_text = "\n".join(hint_texts)
        all_hint_texts.append(hint_text)
    return text, "\n".join(all_hint_texts) if all_hint_texts else ""


def _extract_hints(pull: dict, repo: Repo, issue_number: int) -> list[str]:
    """
    Extract hints from comments associated with a pull request (before first commit)

    Args:
        pull (dict): PR dictionary object from GitHub
        repo (Repo): Repo object
        issue_number (int): issue number
    Return:
        hints (list): list of hints
    """
    # Get all commits in PR
    commits = repo.get_all_loop(
        repo.api.pulls.list_commits, pull_number=pull["number"], quiet=True
    )
    commits = list(commits)
    if len(commits) == 0:
        # If there are no comments, return no hints
        return []
    # Get time of first commit in PR
    commit_time = commits[0].commit.author.date  # str
    commit_time = time.mktime(time.strptime(commit_time, "%Y-%m-%dT%H:%M:%SZ"))
    # Get all comments in PR
    all_comments = repo.get_all_loop(
        repo.api.issues.list_comments, issue_number=issue_number, quiet=True
    )
    all_comments = list(all_comments)
    # Iterate through all comments, only keep comments created before first commit
    comments = list()
    for comment in all_comments:
        comment_time = time.mktime(
            time.strptime(comment.updated_at, "%Y-%m-%dT%H:%M:%SZ")
        )  # use updated_at instead of created_at
        if comment_time < commit_time:
            comments.append(comment)
        else:
            break
        # only include information available before the first commit was created
    # Keep text from comments
    comments = [comment.body for comment in comments]
    return comments


def extract_patches(pull: dict, repo: Repo) -> tuple[str, str]:
    """
    Get patch and test patch from PR

    Args:
        pull (dict): PR dictionary object from GitHub
        repo (Repo): Repo object
    Return:
        patch_change_str (str): gold patch
        patch_test_str (str): test patch
    """
    patch = requests.get(pull["diff_url"]).text
    patch_test = ""
    patch_fix = ""
    for hunk in PatchSet(patch):
        if any(
            test_word in hunk.path for test_word in ["test", "tests", "e2e", "testing"]
        ):
            patch_test += str(hunk)
        else:
            patch_fix += str(hunk)
    return patch_fix, patch_test



import re

from swesmith.constants import TODO_REWRITE, CodeEntity
from tree_sitter import Language, Parser
import tree_sitter_php as tsphp
from swesmith.bug_gen.adapters.utils import build_entity

PHP_LANGUAGE = Language(tsphp.language_php())


class PhpEntity(CodeEntity):
    @property
    def name(self) -> str:
        if self.node.type == "function_definition":
            for child in self.node.children:
                if child.type == "name":
                    return child.text.decode("utf-8")
        elif self.node.type == "method_declaration":
            for child in self.node.children:
                if child.type == "name":
                    func_name = child.text.decode("utf-8")
                    # Find the class this method belongs to
                    class_node = self._find_parent_class()
                    if class_node:
                        class_name = self._get_class_name(class_node)
                        return f"{class_name}::{func_name}" if class_name else func_name
                    return func_name
        elif self.node.type == "class_declaration":
            for child in self.node.children:
                if child.type == "name":
                    return child.text.decode("utf-8")
        return "unknown"

    def _find_parent_class(self):
        """Find the parent class node for a method."""
        current = self.node.parent
        while current:
            if current.type == "class_declaration":
                return current
            current = current.parent
        return None

    def _get_class_name(self, class_node):
        """Extract class name from a class node."""
        for child in class_node.children:
            if child.type == "name":
                return child.text.decode("utf-8")
        return None

    @property
    def signature(self) -> str:
        # Find the opening brace '{' and remove everything after it
        return self.src_code.split("{", 1)[0].strip()

    @property
    def stub(self) -> str:
        # Find the opening brace '{' and remove everything after it
        match = re.search(r"\{", self.src_code)
        if match:
            body_start = match.start()
            return (
                self.src_code[:body_start].rstrip() + " {\n\t// " + TODO_REWRITE + "\n}"
            )
        else:
            # If no body found, return the original code
            return self.src_code


def get_entities_from_file_php(
    entities: list[PhpEntity],
    file_path: str,
    max_entities: int = -1,
) -> list[PhpEntity]:
    """
    Parse a .php file and return up to max_entities top-level functions, methods, and classes.
    If max_entities < 0, collects them all.
    """
    parser = Parser(PHP_LANGUAGE)

    try:
        file_content = open(file_path, "r", encoding="utf8").read()
        tree = parser.parse(bytes(file_content, "utf8"))
        root = tree.root_node
        lines = file_content.splitlines()

        def walk(node):
            # stop if we've hit the limit
            if 0 <= max_entities == len(entities):
                return

            if node.type in [
                "function_definition",
                "method_declaration",
                "class_declaration",
            ]:
                entities.append(build_entity(node, lines, file_path, PhpEntity))
                if 0 <= max_entities == len(entities):
                    return

            for child in node.children:
                walk(child)

        walk(root)
        return entities
    except Exception:
        return entities



"""
Purpose: Given the validation logs, create a SWE-bench-style dataset + set of repositories
that can be run with SWE-agent. Each instances is of the form:

{
    "instance_id":
    "repo":
    "patch":
    "test_patch":
    "problem_statement":
    "FAIL_TO_PASS":
    "PASS_TO_PASS":
    "version":
}

This script will clone the repository, apply the patches and push them to new branches.

IMPORTANT: Make sure you run authenticated git, because else you'll get rate limit issues.

Note: It cannot be strictly SWE-bench. Using SWE-bench styles + infra would be difficult because the
installation specifications are fundamentally different. Therefore, the construction of this
dataset aims for two goals:
* To be runnable in SWE-agent
* To be easy to evaluate with our custom scripts.

Usage: python -m swesmith.harness.gather logs/run_validation/<run_id>
"""

import argparse
import json
import os
import shutil
import subprocess

from pathlib import Path
from swebench.harness.constants import (
    FAIL_TO_PASS,
    PASS_TO_PASS,
    KEY_INSTANCE_ID,
    LOG_REPORT,
)
from swesmith.constants import (
    GIT_APPLY_CMDS,
    KEY_IMAGE_NAME,
    KEY_PATCH,
    KEY_TIMED_OUT,
    LOG_DIR_TASKS,
    LOG_DIR_RUN_VALIDATION,
    REF_SUFFIX,
)
from swesmith.profiles import registry
from tqdm.auto import tqdm

FAILURE_TIPS = """
IMPORTANT

1. If this script fails, you might have to remove the repo & reclone it or remove all branches. 
   Else you might get issues during git checkout -o . 
   Because some branches exist locally but not pushed to the remote on GitHub.

2. Make sure you run authenticated git, because else you'll get rate limit issues that are 
   interpreted as non-existent branches. Causing issues similar to 1.
"""

SUBPROCESS_ARGS = {
    "check": True,
    "shell": True,
}


def main(*args, **kwargs):
    """
    Main entry point for the script.
    """
    try:
        _main(*args, **kwargs)
    except Exception:
        print("=" * 80)
        print("=" * 80)
        print(FAILURE_TIPS)
        print("=" * 80)
        print("=" * 80)
        raise


def skip_print(reason: str, pbar: tqdm, stats: dict, verbose: bool):
    stats["skipped"] += 1
    pbar.set_postfix(stats)
    if verbose:
        print(f"[SKIP] {reason}")
    pbar.update()
    return stats


def check_if_branch_exists(
    repo_name: str,
    subfolder: str,
    main_branch: str,
    override_branch: bool,
    verbose: bool,
):
    branch_exists = None
    try:
        subprocess.run(f"git checkout {subfolder}", cwd=repo_name, **SUBPROCESS_ARGS)
        if override_branch:
            # Delete the branch remotely
            subprocess.run(
                f"git push --delete origin {subfolder}",
                cwd=repo_name,
                **SUBPROCESS_ARGS,
            )
            if verbose:
                print(f"[{subfolder}] Overriding existing branch")
            branch_exists = False
        else:
            branch_exists = True
        subprocess.run(f"git checkout {main_branch}", cwd=repo_name, **SUBPROCESS_ARGS)
        subprocess.run(f"git branch -D {subfolder}", cwd=repo_name, **SUBPROCESS_ARGS)
    except Exception:
        branch_exists = False
        pass
    return branch_exists


def _main(
    validation_logs_path: str | Path,
    *,
    debug_subprocess: bool = False,
    override_branch: bool = False,
    repush_image: bool = False,
    verbose: bool = False,
):
    """
    Create a SWE-bench-style dataset from the validation logs.

    Args:
        validation_logs_path: Path to the validation logs
        debug_subprocess: Whether to output subprocess output
    """
    if not debug_subprocess:
        SUBPROCESS_ARGS["stdout"] = subprocess.DEVNULL
        SUBPROCESS_ARGS["stderr"] = subprocess.DEVNULL

    validation_logs_path = Path(validation_logs_path)
    assert validation_logs_path.resolve().is_relative_to(
        LOG_DIR_RUN_VALIDATION.resolve()
    ), f"Validation logs should be in {LOG_DIR_RUN_VALIDATION}"
    assert validation_logs_path.exists(), (
        f"Validation logs path {validation_logs_path} does not exist"
    )
    assert validation_logs_path.is_dir(), (
        f"Validation logs path {validation_logs_path} is not a directory"
    )

    run_id = validation_logs_path.name
    print(f"{run_id=}")
    task_instances_path = LOG_DIR_TASKS / f"{run_id}.json"
    print(f"Out Path: {task_instances_path}")
    task_instances = []
    created_repos = set()

    completed_ids = []
    subfolders = os.listdir(validation_logs_path)
    if not override_branch and os.path.exists(task_instances_path):
        with open(task_instances_path) as f:
            task_instances = [
                x
                for x in json.load(f)
                if x[KEY_INSTANCE_ID] in subfolders  # Omits removed bugs
            ]
        completed_ids = [x[KEY_INSTANCE_ID] for x in task_instances]
        print(f"Found {len(task_instances)} existing task instances")
        subfolders = [x for x in subfolders if x not in completed_ids]

    stats = {"new_tasks": 0, "skipped": 0}
    print(f"Will process {len(subfolders)} instances")
    pbar = tqdm(subfolders, desc="Conversion", disable=verbose)
    for subfolder in sorted(subfolders):
        if subfolder.endswith(REF_SUFFIX) or subfolder in completed_ids:
            # Skip reference run or instances that have been completed
            stats = skip_print(f"{subfolder}: Reference", pbar, stats, verbose)
            continue

        path_results = os.path.join(validation_logs_path, subfolder, LOG_REPORT)
        path_patch = os.path.join(validation_logs_path, subfolder, "patch.diff")

        if not os.path.exists(path_results):
            stats = skip_print(f"{subfolder}: No results", pbar, stats, verbose)
            continue

        with open(path_results) as f:
            results = json.load(f)
        if FAIL_TO_PASS not in results or PASS_TO_PASS not in results:
            stats = skip_print(
                f"{subfolder}: No validatable bugs", pbar, stats, verbose
            )
            continue

        n_f2p = len(results[FAIL_TO_PASS])
        n_p2p = len(results[PASS_TO_PASS])
        pr_exception = (
            ".pr_" in subfolder and n_p2p == 0 and n_f2p > 0
        )  # TODO: Better way to determine if it's a PR miror?
        if not pr_exception and (KEY_TIMED_OUT in results or n_f2p == 0 or n_p2p == 0):
            # Skip instances that timed out OR don't have F2P or P2P
            stats = skip_print(
                f"{subfolder}: No validatable bugs: {n_f2p=}, {n_p2p=}",
                pbar,
                stats,
                verbose,
            )
            continue

        with open(path_patch) as f:
            patch_content = f.read()
        task_instance = {
            KEY_INSTANCE_ID: subfolder,
            KEY_PATCH: patch_content,
            FAIL_TO_PASS: results[FAIL_TO_PASS],
            PASS_TO_PASS: results[PASS_TO_PASS],
        }
        rp = registry.get_from_inst(task_instance)
        task_instance[KEY_IMAGE_NAME] = rp.image_name
        task_instance["repo"] = rp.mirror_name

        # Clone repository
        _, cloned = rp.clone()
        if cloned:
            created_repos.add(rp.repo_name)
        main_branch = (
            subprocess.run(
                "git rev-parse --abbrev-ref HEAD",
                cwd=rp.repo_name,
                capture_output=True,
                shell=True,
                check=True,
            )
            .stdout.decode()
            .strip()
        )

        # Check if branch already created for this problem
        branch_exists = check_if_branch_exists(
            rp.repo_name, subfolder, main_branch, override_branch, verbose
        )
        if branch_exists:
            task_instances.append(task_instance)
            stats = skip_print(
                f"{subfolder}: Branch `{subfolder}` exists",
                pbar,
                stats,
                verbose,
            )
            continue
        elif verbose:
            print(f"[{subfolder}] Does not exist yet")

        # Apply patch
        applied = False
        for git_apply in GIT_APPLY_CMDS:
            output = subprocess.run(
                f"{git_apply} ../{path_patch}",
                cwd=rp.repo_name,
                capture_output=True,
                shell=True,
            )
            if output.returncode == 0:
                applied = True
                break
            else:
                # Remove any artifacts
                subprocess.run("git reset --hard", cwd=rp.repo_name, **SUBPROCESS_ARGS)
        if not applied:
            raise Exception(f"[{subfolder}] Failed to apply patch to {rp.repo_name}")
        if verbose:
            print(f"[{subfolder}] Bug patch applied successfully")

        # Create a branch, check it out, commit, push the branch, and cleanup
        cmds = [
            "git config user.email 'swesmith@swesmith.ai'",
            "git config user.name 'swesmith'",
            "git config commit.gpgsign false",
            f"git checkout -b {subfolder}",
            "git add .",
            "git commit --no-gpg-sign -m 'Bug Patch'",
        ]
        for cmd in cmds:
            if debug_subprocess:
                print(f"[{subfolder}] {cmd}")
            subprocess.run(cmd, cwd=rp.repo_name, **SUBPROCESS_ARGS)

        # Create test patch by removing F2P test files
        f2p_test_files, _ = rp.get_test_files(task_instance)
        if f2p_test_files:
            # Remove the test files
            for test_file in f2p_test_files:
                test_file_path = os.path.join(rp.repo_name, test_file)
                if os.path.exists(test_file_path):
                    os.remove(test_file_path)
                    if verbose:
                        print(f"[{subfolder}] Removed F2P test file: {test_file}")

            # Add and commit removal
            cmds = [
                "git add .",
                "git commit --no-gpg-sign -m 'Remove F2P Tests'",
            ]
            for cmd in cmds:
                if debug_subprocess:
                    print(f"[{subfolder}] {cmd}")
                subprocess.run(cmd, cwd=rp.repo_name, **SUBPROCESS_ARGS)
            if verbose:
                print(f"[{subfolder}] Commit F2P test file(s) removal")
        elif verbose:
            print(f"[{subfolder}] No test files to remove")

        cmds = [
            f"git push origin {subfolder}",
            f"git checkout {main_branch}",
            "git reset --hard",
            f"git branch -D {subfolder}",
        ]
        for cmd in cmds:
            if debug_subprocess:
                print(f"[{subfolder}] {cmd}")
            subprocess.run(cmd, cwd=rp.repo_name, **SUBPROCESS_ARGS)
        if verbose:
            print(f"[{subfolder}] Bug @ branch `{subfolder}`")

        task_instances.append(task_instance)
        if verbose:
            print(f"[{subfolder}] Created task instance")
        stats["new_tasks"] += 1
        pbar.update()

    pbar.close()
    if len(created_repos) > 0:
        print("Cleaning up...")
        for repo in created_repos:
            shutil.rmtree(repo)
            print(f"[{repo}] Removed local clone")
            if repush_image:
                print(f"[{repo}] Rebuilding + pushing image")
                registry.get(repo).push_image(rebuild_image=True)

    task_instances_path.parent.mkdir(parents=True, exist_ok=True)
    with open(task_instances_path, "w") as f:
        json.dump(task_instances, f, indent=4)
    print(f"Wrote {len(task_instances)} instances to {task_instances_path}")
    print(f"- {stats['skipped']} skipped")
    print(f"- {stats['new_tasks']} new instances")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Convert validation logs to SWE-bench style dataset"
    )
    parser.add_argument(
        "validation_logs_path", type=str, help="Path to the validation logs"
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Verbose mode",
    )
    # Override branch takes effect when
    # - A branch for the bug already exists
    # - But the local version of the bug (in logs/run_validation) has been modified (out of sync with the branch)
    # In this case, we delete the branch and recreate the bug.
    # This is useful for if you've regenerated a bug, it's validated, and you'd like to override the existing branch.
    parser.add_argument(
        "-o",
        "--override_branch",
        action="store_true",
        help="Override existing branches",
    )
    parser.add_argument(
        "-d",
        "--debug_subprocess",
        action="store_true",
        help="Debug mode (output subprocess output)",
    )
    parser.add_argument(
        "-p",
        "--repush_image",
        action="store_true",
        help="Rebuild and push Docker image for repos (such that latest branches are included)",
    )
    args = parser.parse_args()

    main(**vars(args))



"""
Utility functions for language adapters.
"""

import re
from typing import TypeVar, Type
from swesmith.constants import CodeEntity

T = TypeVar("T", bound=CodeEntity)


def build_entity(
    node,
    lines: list[str],
    file_path: str,
    entity_class: Type[T],
    default_indent_size: int = 4,
) -> T:
    """
    Turns a Tree-sitter node into a CodeEntity object.
    """
    # start_point/end_point are (row, col) zero-based
    start_row, _ = node.start_point
    end_row, _ = node.end_point

    # slice out the raw lines
    snippet = lines[start_row : end_row + 1]

    # detect indent on first line
    first = snippet[0] if snippet else ""
    m = re.match(r"^(?P<indent>[\t ]*)", first)
    indent_str = m.group("indent") if m else ""
    # tabs count as size=1, else use count of spaces, fallback to default_indent_size
    indent_size = 1 if "\t" in indent_str else (len(indent_str) or default_indent_size)
    indent_level = len(indent_str) // indent_size

    # dedent each line
    dedented = []
    for line in snippet:
        if len(line) >= indent_level * indent_size:
            dedented.append(line[indent_level * indent_size :])
        else:
            dedented.append(line.lstrip("\t "))

    return entity_class(
        file_path=file_path,
        indent_level=indent_level,
        indent_size=indent_size,
        line_start=start_row + 1,
        line_end=end_row + 1,
        node=node,
        src_code="\n".join(dedented),
    )



"""
Utility functions for transforming SWE-agent trajectories to fine-tuning format.
"""

import json
import yaml

from swesmith import REPO_DIR

XML_STR_REPLACES = ["old_str", "new_str", "file_text"]


SYSTEM_PROMPT = yaml.safe_load(
    (REPO_DIR / "agent" / "swesmith_infer.yaml").read_text()
)["agent"]["templates"]["system_template"]


def get_messages(traj: dict) -> list[dict]:
    """Extract messages from a swe-agent trajectory.

    We assume that the messages of the last step correspond to the
    full message history.
    This is a bit of an approximation (e.g., requeries after blocked actions
    aren't fully captured)
    """
    last_step = traj["trajectory"][-1]
    # There was a change in output formats in swe-agent 1.1.0:
    # https://swe-agent.com/latest/usage/trajectories/
    # For < 1.1.0, we had the 'messages' field that included messages
    # _after_ the message was performed (and then we remove the last message because
    # it contains the submit/patch)
    # For >= 1.1.0, we have the 'query' field that includes messages that were the
    # direct input to the agent at that step (so do not need to exclude the last message)
    if "messages" in last_step:
        return last_step["messages"][:-1]
    else:
        if last_step["response"] in [
            "Exit due to cost limit",
            "Exit due to context window",
        ]:
            return traj["trajectory"][-2]["query"][:]
        return last_step["query"][:]


def transform_traj_backticks(traj: dict) -> dict:
    """Transform a swe-agent trajectory to backticks format, i.e.,
    for use with the `thought-action` parser of swe-agent where actions
    are extracted from triple-backticks blocks.
    """
    new_traj = []
    for message in get_messages(traj):
        role = message["role"] if message["role"] != "tool" else "user"
        if message["role"] == "assistant":
            content = f"{message['thought']}\n\n```\n{message['action']}\n```"
        elif message["role"] == "system":
            content = message["content"]
        else:
            assert len(message["content"]) == 1
            content = message["content"][0]["text"]
        new_traj.append({"role": role, "content": content})
    return {"messages": new_traj}


def tool_call_to_action(tool_calls: None | list[dict]) -> list[str]:
    actions = []
    if tool_calls is None:
        return []
    for tool_call in tool_calls:
        action = [f"<function={tool_call['function']['name']}>"]
        arguments = json.loads(tool_call["function"]["arguments"])
        for k, v in arguments.items():
            a = f"<parameter={k}>{v}</parameter>"
            if k in XML_STR_REPLACES:
                a = f"<parameter={k}>\n{v}\n</parameter>"
            action.append(a)
        action.append("</function>")
        actions.append("\n".join(action))
    return actions


def transform_traj_xml(traj: dict) -> dict:
    new_traj = []
    for message in get_messages(traj):
        role = message["role"] if message["role"] != "tool" else "user"
        if message["role"] == "assistant":
            if message["content"] == "Exit due to cost limit":
                content = (
                    "Since we have successfully fixed the issue and verified it works, "
                    + "let's submit the changes:\n\n"
                    + "<function=submit>\n</function>"
                )
            else:
                content = message.get("thought", message["content"])
                if "tool_calls" in message:
                    action = "\n".join(tool_call_to_action(message["tool_calls"]))
                    content += f"\n\n{action}"
                content = content.strip()
        elif message["role"] == "system":
            # We replace the system prompt that was used for generating the training trajectories
            # with the system prompt that SWE-agent-LM will use for inference.
            content = SYSTEM_PROMPT
        else:
            if isinstance(message["content"], list):
                assert len(message["content"]) == 1
                content = message["content"][0]["text"]
            elif isinstance(message["content"], str):
                content = message["content"]
            else:
                raise ValueError(f"Message type not recognized: {type(message)}")
        new_traj.append({"role": role, "content": content})
    return {"messages": new_traj}


def transform_traj_toolcalls(traj: dict) -> dict:
    return {"messages": get_messages(traj)}


MAP_STYLE_TO_FUNC = {
    "ticks": transform_traj_backticks,
    "tool": transform_traj_toolcalls,
    "xml": transform_traj_xml,
}



import docker
import re
from dataclasses import dataclass, field

from pathlib import Path
from swebench.harness.constants import (
    FAIL_TO_PASS,
    PASS_TO_PASS,
    KEY_INSTANCE_ID,
    TestStatus,
)
from swebench.harness.docker_build import build_image as build_image_sweb
from swebench.harness.dockerfiles import get_dockerfile_env
from swesmith.constants import LOG_DIR_ENV, ENV_NAME, INSTANCE_REF, ORG_NAME_DH
from swesmith.profiles.base import RepoProfile, registry
from swesmith.profiles.utils import INSTALL_BAZEL, INSTALL_CMAKE


@dataclass
class PythonProfile(RepoProfile):
    """
    Profile for Python repositories.

    This class provides Python-specific defaults and functionality for
    repository profiles, including Python version management and common
    Python installation/test patterns.
    """

    python_version: str = "3.10"
    install_cmds: list[str] = field(
        default_factory=lambda: ["python -m pip install -e ."]
    )
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "pytest --disable-warnings --color=no --tb=no --verbose"
    )
    exts: list[str] = field(default_factory=lambda: [".py"])

    def get_test_files(self, instance: dict) -> tuple[list[str], list[str]]:
        assert FAIL_TO_PASS in instance and PASS_TO_PASS in instance, (
            f"Instance {instance[KEY_INSTANCE_ID]} missing required keys {FAIL_TO_PASS} or {PASS_TO_PASS}"
        )
        _helper = lambda tests: sorted(list(set([x.split("::", 1)[0] for x in tests])))
        return _helper(instance[FAIL_TO_PASS]), _helper(instance[PASS_TO_PASS])

    def build_image(self):
        BASE_IMAGE_KEY = f"{ORG_NAME_DH}/swesmith.x86_64"
        HEREDOC_DELIMITER = "EOF_59812759871"
        PATH_TO_REQS = "swesmith_environment.yml"

        client = docker.from_env()
        with open(self._env_yml) as f:
            reqs = f.read()

        setup_commands = [
            "#!/bin/bash",
            "set -euxo pipefail",
            f"git clone -o origin https://github.com/{self.mirror_name} /{ENV_NAME}",
            f"cd /{ENV_NAME}",
            "source /opt/miniconda3/bin/activate",
            f"cat <<'{HEREDOC_DELIMITER}' > {PATH_TO_REQS}\n{reqs}\n{HEREDOC_DELIMITER}",
            f"conda env create --file {PATH_TO_REQS}",
            f"conda activate {ENV_NAME} && conda install python={self.python_version} -y",
            f"rm {PATH_TO_REQS}",
            f"conda activate {ENV_NAME}",
            'echo "Current environment: $CONDA_DEFAULT_ENV"',
        ] + self.install_cmds
        dockerfile = get_dockerfile_env(
            self.pltf, self.arch, "py", base_image_key=BASE_IMAGE_KEY
        )

        build_image_sweb(
            image_name=self.image_name,
            setup_scripts={"setup_env.sh": "\n".join(setup_commands) + "\n"},
            dockerfile=dockerfile,
            platform=self.pltf,
            client=client,
            build_dir=LOG_DIR_ENV / self.repo_name,
        )

    def log_parser(self, log: str) -> dict[str, str]:
        """Parser for test logs generated with PyTest framework"""
        test_status_map = {}
        for line in log.split("\n"):
            for status in TestStatus:
                is_match = re.match(rf"^(\S+)(\s+){status.value}", line)
                if is_match:
                    test_status_map[is_match.group(1)] = status.value
                    continue
        return test_status_map

    @property
    def _env_yml(self) -> Path:
        return LOG_DIR_ENV / self.repo_name / f"sweenv_{self.repo_name}.yml"


### MARK: Repository Profile Classes ###


@dataclass
class Addict75284f95(PythonProfile):
    owner: str = "mewwts"
    repo: str = "addict"
    commit: str = "75284f9593dfb929cadd900aff9e35e7c7aec54b"


@dataclass
class AliveProgress35853799(PythonProfile):
    owner: str = "rsalmei"
    repo: str = "alive-progress"
    commit: str = "35853799b84ee682af121f7bc5967bd9b62e34c4"


@dataclass
class Apispec8b421526(PythonProfile):
    owner: str = "marshmallow-code"
    repo: str = "apispec"
    commit: str = "8b421526ea1015046de42599dd93da6a3473fe44"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[dev]"])


@dataclass
class Arrow1d70d009(PythonProfile):
    owner: str = "arrow-py"
    repo: str = "arrow"
    commit: str = "1d70d0091980ea489a64fa95a48e99b45f29f0e7"


@dataclass
class AstroidB114f6b5(PythonProfile):
    owner: str = "pylint-dev"
    repo: str = "astroid"
    commit: str = "b114f6b58e749b8ab47f80490dce73ea80d8015f"


@dataclass
class AsyncTimeoutD0baa9f1(PythonProfile):
    owner: str = "aio-libs"
    repo: str = "async-timeout"
    commit: str = "d0baa9f162b866e91881ae6cfa4d68839de96fb5"


@dataclass
class AutogradAc044f0d(PythonProfile):
    owner: str = "HIPS"
    repo: str = "autograd"
    commit: str = "ac044f0de1185b725955595840135e9ade06aaed"
    install_cmds: list = field(
        default_factory=lambda: ["pip install -e '.[scipy,test]'"]
    )

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        for line in log.split("\n"):
            for status in TestStatus:
                is_match = re.match(rf"^\[gw\d\]\s{status.value}\s(\S+)", line)
                if is_match:
                    test_status_map[is_match.group(1)] = status.value
                    continue
        return test_status_map


@dataclass
class Bleach73871d76(PythonProfile):
    owner: str = "mozilla"
    repo: str = "bleach"
    commit: str = "73871d766de1e33a296eeb4f9faf2451f28bee39"


@dataclass
class Boltons3bfcfdd0(PythonProfile):
    owner: str = "mahmoud"
    repo: str = "boltons"
    commit: str = "3bfcfdd04395b6cc74a5c0cdc72c8f64cc4ac01f"


@dataclass
class Bottlea8dfef30(PythonProfile):
    owner: str = "bottlepy"
    repo: str = "bottle"
    commit: str = "a8dfef301dec35f13e7578306002c40796651629"


@dataclass
class Cantools0c6a7871(PythonProfile):
    owner: str = "cantools"
    repo: str = "cantools"
    commit: str = "0c6a78711409e4307de34582f795ddb426d58dd8"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[dev,plot]"])


@dataclass
class ChannelsA144b4b8(PythonProfile):
    owner: str = "django"
    repo: str = "channels"
    commit: str = "a144b4b8881a93faa567a6bdf2d7f518f4c16cd2"
    install_cmds: list = field(
        default_factory=lambda: ["pip install -e .[tests,daphne]"]
    )


@dataclass
class Chardet9630f238(PythonProfile):
    owner: str = "chardet"
    repo: str = "chardet"
    commit: str = "9630f2382faa50b81be2f96fd3dfab5f6739a0ef"


@dataclass
class CharsetNormalizer1fdd6463(PythonProfile):
    owner: str = "jawah"
    repo: str = "charset_normalizer"
    commit: str = "1fdd64633572040ab60e62e8b24f29cb7e17660b"


@dataclass
class ClickFde47b4b4(PythonProfile):
    owner: str = "pallets"
    repo: str = "click"
    commit: str = "fde47b4b4f978f179b9dff34583cb2b99021f482"


@dataclass
class Cloudpickle6220b0ce(PythonProfile):
    owner: str = "cloudpipe"
    repo: str = "cloudpickle"
    commit: str = "6220b0ce83ffee5e47e06770a1ee38ca9e47c850"


@dataclass
class PythonColorlogDfa10f59(PythonProfile):
    owner: str = "borntyping"
    repo: str = "python-colorlog"
    commit: str = "dfa10f59186d3d716aec4165ee79e58f2265c0eb"


@dataclass
class CookiecutterB4451231(PythonProfile):
    owner: str = "cookiecutter"
    repo: str = "cookiecutter"
    commit: str = "b4451231809fb9e4fc2a1e95d433cb030e4b9e06"


@dataclass
class Daphne32ac73e1(PythonProfile):
    owner: str = "django"
    repo: str = "daphne"
    commit: str = "32ac73e1a0fb87af0e3280c89fe4cc3ff1231b37"


@dataclass
class Dataset5c2dc8d3(PythonProfile):
    owner: str = "pudo"
    repo: str = "dataset"
    commit: str = "5c2dc8d3af1e0af0290dcd7ae2cae92589f305a1"
    install_cmds: list = field(default_factory=lambda: ["python setup.py install"])


@dataclass
class DeepdiffEd252022(PythonProfile):
    owner: str = "seperman"
    repo: str = "deepdiff"
    commit: str = "ed2520229d0369813f6e54cdf9c7e68e8073ef62"
    install_cmds: list = field(
        default_factory=lambda: [
            "pip install -r requirements-dev.txt",
            "pip install -e .",
        ]
    )


@dataclass
class DjangoMoney835c1ab8(PythonProfile):
    owner: str = "django-money"
    repo: str = "django-money"
    commit: str = "835c1ab867d11137b964b94936692bea67a038ec"
    install_cmds: list = field(
        default_factory=lambda: ["pip install -e .[test,exchange]"]
    )


@dataclass
class Dominate9082227e(PythonProfile):
    owner: str = "Knio"
    repo: str = "dominate"
    commit: str = "9082227e93f5a370012bb934286caf7385d3e7ac"


@dataclass
class PythonDotenv2b8635b7(PythonProfile):
    owner: str = "theskumar"
    repo: str = "python-dotenv"
    commit: str = "2b8635b79f1aa15cade0950117d4e7d12c298766"


@dataclass
class DrfNestedRouters6144169d(PythonProfile):
    owner: str = "alanjds"
    repo: str = "drf-nested-routers"
    commit: str = "6144169d5c33a1c5134b2fedac1d6cfa312c174e"
    install_cmds: list = field(
        default_factory=lambda: ["pip install -r requirements.txt", "pip install -e ."]
    )


@dataclass
class Environs73c372df(PythonProfile):
    owner: str = "sloria"
    repo: str = "environs"
    commit: str = "73c372df71002312615ad0349ae11274bb3edc69"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[dev]"])


@dataclass
class Exceptiongroup0b4f4937(PythonProfile):
    owner: str = "agronholm"
    repo: str = "exceptiongroup"
    commit: str = "0b4f49378b585a338ae10abd72ec2006c5057d7b"


@dataclass
class Faker8b401a7d(PythonProfile):
    owner: str = "joke2k"
    repo: str = "faker"
    commit: str = "8b401a7d68f5fda1276f36a8fc502ef32050ed72"


@dataclass
class FeedparserCad965a3(PythonProfile):
    owner: str = "kurtmckee"
    repo: str = "feedparser"
    commit: str = "cad965a3f52c4b077221a2142fb14ef7f68cd576"


@dataclass
class Flake8Cf1542ce(PythonProfile):
    owner: str = "PyCQA"
    repo: str = "flake8"
    commit: str = "cf1542cefa3e766670b2066dd75c4571d682a649"


@dataclass
class FlashtextB316c7e9(PythonProfile):
    owner: str = "vi3k6i5"
    repo: str = "flashtext"
    commit: str = "b316c7e9e54b6b4d078462b302a83db85f884a94"


@dataclass
class FlaskBc098406(PythonProfile):
    owner: str = "pallets"
    repo: str = "flask"
    commit: str = "bc098406af9537aacc436cb2ea777fbc9ff4c5aa"
    eval_sets: set[str] = field(
        default_factory=lambda: {
            "SWE-bench/SWE-bench",
            "SWE-bench/SWE-bench_Lite",
            "SWE-bench/SWE-bench_Verified",
        }
    )


@dataclass
class Freezegun5f171db0(PythonProfile):
    owner: str = "spulec"
    repo: str = "freezegun"
    commit: str = "5f171db0aaa02c4ade003bbc8885e0bb19efbc81"


@dataclass
class Funcy207a7810(PythonProfile):
    owner: str = "Suor"
    repo: str = "funcy"
    commit: str = "207a7810c216c7408596d463d3f429686e83b871"


@dataclass
class FurlDa386f68(PythonProfile):
    owner: str = "gruns"
    repo: str = "furl"
    commit: str = "da386f68b8d077086c25adfd205a4c3d502c3012"


@dataclass
class FvcoreA491d5b9(PythonProfile):
    owner: str = "facebookresearch"
    repo: str = "fvcore"
    commit: str = "a491d5b9a06746f387aca2f1f9c7c7f28e20bef9"
    install_cmds: list = field(
        default_factory=lambda: [
            "pip install torch shapely",
            "rm tests/test_focal_loss.py",
            "pip install -e .",
        ]
    )


@dataclass
class GlomFb3c4e76(PythonProfile):
    owner: str = "mahmoud"
    repo: str = "glom"
    commit: str = "fb3c4e76f28816aebfd2538980e617742e98a7c2"


@dataclass
class Gpxpy09fc46b3(PythonProfile):
    owner: str = "tkrajina"
    repo: str = "gpxpy"
    commit: str = "09fc46b3cad16b5bf49edf8e7ae873794a959620"
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "pytest test.py --verbose --color=no --tb=no --disable-warnings"
    )


@dataclass
class Grafanalib5c3b17ed(PythonProfile):
    owner: str = "weaveworks"
    repo: str = "grafanalib"
    commit: str = "5c3b17edaa437f0bc09b5f1b9275dc8fb91689fb"


@dataclass
class Graphene82903263(PythonProfile):
    owner: str = "graphql-python"
    repo: str = "graphene"
    commit: str = "82903263080b3b7f22c2ad84319584d7a3b1a1f6"


@dataclass
class GspreadA8be3b96(PythonProfile):
    owner: str = "burnash"
    repo: str = "gspread"
    commit: str = "a8be3b96f9276779ab680d84a0982282fb184000"


@dataclass
class GTTSDbcda4f39(PythonProfile):
    owner: str = "pndurette"
    repo: str = "gTTS"
    commit: str = "dbcda4f396074427172d4a1f798a172686ace6e0"


@dataclass
class GunicornBacbf8aa(PythonProfile):
    owner: str = "benoitc"
    repo: str = "gunicorn"
    commit: str = "bacbf8aa5152b94e44aa5d2a94aeaf0318a85248"


@dataclass
class H11Bed0dd4ae(PythonProfile):
    owner: str = "python-hyper"
    repo: str = "h11"
    commit: str = "bed0dd4ae9774b962b19833941bb9ec4dc403da9"


@dataclass
class IcecreamF76fef56(PythonProfile):
    owner: str = "gruns"
    repo: str = "icecream"
    commit: str = "f76fef56b66b59fd9a89502c60a99fbe28ee36bd"


@dataclass
class InflectC079a96a(PythonProfile):
    owner: str = "jaraco"
    repo: str = "inflect"
    commit: str = "c079a96a573ece60b54bd5210bb0f414beb74dcd"


@dataclass
class Iniconfig16793ead(PythonProfile):
    owner: str = "pytest-dev"
    repo: str = "iniconfig"
    commit: str = "16793eaddac67de0b8d621ae4e42e05b927e8d67"


@dataclass
class Isodate17cb25eb(PythonProfile):
    owner: str = "gweis"
    repo: str = "isodate"
    commit: str = "17cb25eb7bc3556a68f3f7b241313e9bb8b23760"


@dataclass
class JinjaAda0a9a6(PythonProfile):
    owner: str = "pallets"
    repo: str = "jinja"
    commit: str = "ada0a9a6fc265128b46949b5144d2eaa55e6df2c"


@dataclass
class Jsonschema93e0caa5(PythonProfile):
    owner: str = "python-jsonschema"
    repo: str = "jsonschema"
    commit: str = "93e0caa5752947ec77333da81a634afe41a022ed"


@dataclass
class LangdetectA1598f1a(PythonProfile):
    owner: str = "Mimino666"
    repo: str = "langdetect"
    commit: str = "a1598f1afcbfe9a758cfd06bd688fbc5780177b2"


@dataclass
class LineProfilerA646bf0f(PythonProfile):
    owner: str = "pyutils"
    repo: str = "line_profiler"
    commit: str = "a646bf0f9ab3d15264a1be14d0d4ee6894966f6a"


@dataclass
class PythonMarkdownify6258f5c3(PythonProfile):
    owner: str = "matthewwithanm"
    repo: str = "python-markdownify"
    commit: str = "6258f5c38b97ab443b4ddf03e6676ce29b392d06"


@dataclass
class Markupsafe620c06c9(PythonProfile):
    owner: str = "pallets"
    repo: str = "markupsafe"
    commit: str = "620c06c919c1bd7bb1ce3dbee402e1c0c56e7ac3"


@dataclass
class Marshmallow9716fc62(PythonProfile):
    owner: str = "marshmallow-code"
    repo: str = "marshmallow"
    commit: str = "9716fc629976c9d3ce30cd15d270d9ac235eb725"


@dataclass
class MidoA0158ff9(PythonProfile):
    owner: str = "mido"
    repo: str = "mido"
    commit: str = "a0158ff95a08f9a4eef628a2e7c793fd3a466640"
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "pytest --disable-warnings --color=no --tb=no --verbose -rs -c /dev/null"
    )

    def get_test_files(self, instance: dict) -> list[str]:
        f2p_files, p2p_files = super().get_test_files(instance)
        prefix = "../dev/"
        _helper = (
            lambda test_file: test_file[len(prefix) :]
            if test_file.startswith(prefix)
            else test_file
        )
        remove_prefix = lambda test_files: sorted(list(set(map(_helper, test_files))))
        return remove_prefix(f2p_files), remove_prefix(p2p_files)


@dataclass
class MistuneBf54ef67(PythonProfile):
    owner: str = "lepture"
    repo: str = "mistune"
    commit: str = "bf54ef67390e02a5cdee7495d4386d7770c1902b"


@dataclass
class Nikola0f4c230e(PythonProfile):
    owner: str = "getnikola"
    repo: str = "nikola"
    commit: str = "0f4c230e5159e4e937463eb8d6d2ddfcbb09def2"
    install_cmds: list = field(
        default_factory=lambda: ["pip install -e '.[extras,tests]'"]
    )


@dataclass
class Oauthlib1fd52536(PythonProfile):
    owner: str = "oauthlib"
    repo: str = "oauthlib"
    commit: str = "1fd5253630c03e3f12719dd8c13d43111f66a8d2"


@dataclass
class Paramiko23f92003(PythonProfile):
    owner: str = "paramiko"
    repo: str = "paramiko"
    commit: str = "23f92003898b060df0e2b8b1d889455264e63a3e"
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "pytest -rA --color=no --disable-warnings"
    )

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        for line in log.split("\n"):
            for status in TestStatus:
                is_match = re.match(rf"^{status.value}\s(\S+)", line)
                if is_match:
                    test_status_map[is_match.group(1)] = status.value
                    continue
        return test_status_map


@dataclass
class Parse30da9e4f(PythonProfile):
    owner: str = "r1chardj0n3s"
    repo: str = "parse"
    commit: str = "30da9e4f37fdd979487c9fe2673df35b6b204c72"


@dataclass
class Parsimonious0d3f5f93(PythonProfile):
    owner: str = "erikrose"
    repo: str = "parsimonious"
    commit: str = "0d3f5f93c98ae55707f0958366900275d1ce094f"


@dataclass
class Parso338a5760(PythonProfile):
    owner: str = "davidhalter"
    repo: str = "parso"
    commit: str = "338a57602740ad0645b2881e8c105ffdc959e90d"
    install_cmds: list = field(default_factory=lambda: ["python setup.py install"])


@dataclass
class PatsyA5d16484(PythonProfile):
    owner: str = "pydata"
    repo: str = "patsy"
    commit: str = "a5d1648401b0ea0649b077f4b98da27db947d2d0"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[test]"])


@dataclass
class PdfminerSix1a8bd2f7(PythonProfile):
    owner: str = "pdfminer"
    repo: str = "pdfminer.six"
    commit: str = "1a8bd2f730295b31d6165e4d95fcb5a03793c978"


@dataclass
class Pdfplumber02ff4313(PythonProfile):
    owner: str = "jsvine"
    repo: str = "pdfplumber"
    commit: str = "02ff4313f846380fefccec9c73fb4c8d8a80d0ee"
    install_cmds: list = field(
        default_factory=lambda: [
            "apt-get update && apt-get install ghostscript -y",
            "pip install -e .",
        ]
    )


@dataclass
class PipdeptreeC31b6418(PythonProfile):
    owner: str = "tox-dev"
    repo: str = "pipdeptree"
    commit: str = "c31b641817f8235df97adf178ffd8e4426585f7a"
    install_cmds: list = field(
        default_factory=lambda: [
            "apt-get update && apt-get install graphviz -y",
            "pip install -e .[test,graphviz]",
        ]
    )


@dataclass
class PrettytableCa90b055(PythonProfile):
    owner: str = "prettytable"
    repo: str = "prettytable"
    commit: str = "ca90b055f20a6e8a06dcc46c2e3afe8ff1e8d0f1"


@dataclass
class Ptyprocess1067dbda(PythonProfile):
    owner: str = "pexpect"
    repo: str = "ptyprocess"
    commit: str = "1067dbdaf5cc3ab4786ae355aba7b9512a798734"


@dataclass
class Pyasn10f07d724(PythonProfile):
    owner: str = "pyasn1"
    repo: str = "pyasn1"
    commit: str = "0f07d7242a78ab4d129b26256d7474f7168cf536"


@dataclass
class Pydicom7d361b3d(PythonProfile):
    owner: str = "pydicom"
    repo: str = "pydicom"
    commit: str = "7d361b3d764dbbb1f8ad7af015e80ce96f6bf286"
    python_version: str = "3.11"


@dataclass
class PyfigletF8c5f35b(PythonProfile):
    owner: str = "pwaller"
    repo: str = "pyfiglet"
    commit: str = "f8c5f35be70a4bbf93ac032334311b326bc61688"


@dataclass
class Pygments27649ebbf(PythonProfile):
    owner: str = "pygments"
    repo: str = "pygments"
    commit: str = "27649ebbf5a2519725036b48ec99ef7745f100af"


@dataclass
class Pyopenssl04766a49(PythonProfile):
    owner: str = "pyca"
    repo: str = "pyopenssl"
    commit: str = "04766a496eb11f69f6226a5a0dfca4db90a5cbd1"


@dataclass
class Pyparsing533adf47(PythonProfile):
    owner: str = "pyparsing"
    repo: str = "pyparsing"
    commit: str = "533adf471f85b570006871e60a2e585fcda5b085"


@dataclass
class Pypika1c9646f0(PythonProfile):
    owner: str = "kayak"
    repo: str = "pypika"
    commit: str = "1c9646f0a019a167c32b649b6f5e6423c5ba2c9b"


@dataclass
class Pyquery811cd048(PythonProfile):
    owner: str = "gawel"
    repo: str = "pyquery"
    commit: str = "811cd048ffbe4e69fdc512863671131f98d691fb"


@dataclass
class PySnooper57472b46(PythonProfile):
    owner: str = "cool-RR"
    repo: str = "PySnooper"
    commit: str = "57472b4677b6c041647950f28f2d5750c38326c6"


@dataclass
class PythonDocx0cf6d71f(PythonProfile):
    owner: str = "python-openxml"
    repo: str = "python-docx"
    commit: str = "0cf6d71fb47ede07ecd5de2a8655f9f46c5f083d"


@dataclass
class PythonJsonLogger5f85723f(PythonProfile):
    owner: str = "madzak"
    repo: str = "python-json-logger"
    commit: str = "5f85723f4693c7289724fdcda84cfc0b62da74d4"


@dataclass
class PythonPinyinE42dede5(PythonProfile):
    owner: str = "mozillazg"
    repo: str = "python-pinyin"
    commit: str = "e42dede51abbc40e225da9a8ec8e5bd0043eed21"


@dataclass
class PythonPptx278b47b1(PythonProfile):
    owner: str = "scanny"
    repo: str = "python-pptx"
    commit: str = "278b47b1dedd5b46ee84c286e77cdfb0bf4594be"


@dataclass
class PythonQrcode456b01d4(PythonProfile):
    owner: str = "lincolnloop"
    repo: str = "python-qrcode"
    commit: str = "456b01d41f16e0cfb0f70c687848e276b78c3e8a"


@dataclass
class PythonReadability40256f40(PythonProfile):
    owner: str = "buriy"
    repo: str = "python-readability"
    commit: str = "40256f40389c1f97be5e83d7838547581653c6aa"


@dataclass
class PythonSlugify872b3750(PythonProfile):
    owner: str = "un33k"
    repo: str = "python-slugify"
    commit: str = "872b37509399a7f02e53f46ad9881f63f66d334b"
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "python test.py --verbose"
    )

    def get_test_files(self, instance: dict) -> list[str]:
        return ["test.py"], ["test.py"]

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        pattern = r"^([a-zA-Z0-9_\-,\.\s\(\)']+)\s\.{3}\s"
        for line in log.split("\n"):
            is_match = re.match(f"{pattern}ok$", line)
            if is_match:
                test_status_map[is_match.group(1)] = TestStatus.PASSED.value
                continue
            for keyword, status in {
                "FAIL": TestStatus.FAILED,
                "ERROR": TestStatus.ERROR,
            }.items():
                is_match = re.match(f"{pattern}{keyword}$", line)
                if is_match:
                    test_status_map[is_match.group(1)] = status.value
                    continue
        return test_status_map


@dataclass
class Radon54b88e58(PythonProfile):
    owner: str = "rubik"
    repo: str = "radon"
    commit: str = "54b88e5878b2724bf4d77f97349588b811abdff2"


@dataclass
class Records5941ab27(PythonProfile):
    owner: str = "kennethreitz"
    repo: str = "records"
    commit: str = "5941ab2798cb91455b6424a9564c9cd680475fbe"


@dataclass
class RedDiscordBot33e0eac7(PythonProfile):
    owner: str = "Cog-Creators"
    repo: str = "Red-DiscordBot"
    commit: str = "33e0eac741955ce5b7e89d9b8f2f2712727af770"


@dataclass
class Result0b855e1e(PythonProfile):
    owner: str = "rustedpy"
    repo: str = "result"
    commit: str = "0b855e1e38a08d6f0a4b0138b10c127c01e54ab4"


@dataclass
class Safety7654596b(PythonProfile):
    owner: str = "pyupio"
    repo: str = "safety"
    commit: str = "7654596be933f8310b294dbc85a7af6066d06e4f"


@dataclass
class Scrapy35212ec5(PythonProfile):
    owner: str = "scrapy"
    repo: str = "scrapy"
    commit: str = "35212ec5b05a3af14c9f87a6193ab24e33d62f9f"
    install_cmds: list = field(
        default_factory=lambda: [
            "apt-get update && apt-get install -y libxml2-dev libxslt-dev libjpeg-dev",
            "python -m pip install -e .",
            "rm tests/test_feedexport.py",
            "rm tests/test_pipeline_files.py",
        ]
    )
    min_testing: bool = True


@dataclass
class Schedule82a43db1(PythonProfile):
    owner: str = "dbader"
    repo: str = "schedule"
    commit: str = "82a43db1b938d8fdf60103bd41f329e06c8d3651"


@dataclass
class Schema24a30457(PythonProfile):
    owner: str = "keleshev"
    repo: str = "schema"
    commit: str = "24a3045773eac497c659f24b32f24a281be9f286"


@dataclass
class SoupsieveA8080d97(PythonProfile):
    owner: str = "facelessuser"
    repo: str = "soupsieve"
    commit: str = "a8080d97a0355e316981cb0c5c887a861c4244e3"


@dataclass
class Sqlfluff50a1c4b6(PythonProfile):
    owner: str = "sqlfluff"
    repo: str = "sqlfluff"
    commit: str = "50a1c4b6ff171188b6b70b39afe82a707b4919ac"
    min_testing: bool = True


@dataclass
class Sqlglot036601ba(PythonProfile):
    owner: str = "tobymao"
    repo: str = "sqlglot"
    commit: str = "036601ba9cbe4d175d6a9d38bc27587eab858968"
    install_cmds: list = field(default_factory=lambda: ['pip install -e ".[dev]"'])
    min_testing: bool = True


@dataclass
class SqlparseE57923b3(PythonProfile):
    owner: str = "andialbrecht"
    repo: str = "sqlparse"
    commit: str = "e57923b3aa823c524c807953cecc48cf6eec2cb2"


@dataclass
class Stackprinter219fcc52(PythonProfile):
    owner: str = "cknd"
    repo: str = "stackprinter"
    commit: str = "219fcc522fa5fd6e440703358f6eb408f3ffc007"


@dataclass
class StarletteDb5063c2(PythonProfile):
    owner: str = "encode"
    repo: str = "starlette"
    commit: str = "db5063c26030e019f7ee62aef9a1b564eca9f1d6"


@dataclass
class PythonStringSimilarity115acaac(PythonProfile):
    owner: str = "luozhouyang"
    repo: str = "python-string-similarity"
    commit: str = "115acaacf926b41a15664bd34e763d074682bda3"


@dataclass
class SunpyF8edfd5c(PythonProfile):
    owner: str = "sunpy"
    repo: str = "sunpy"
    commit: str = "f8edfd5c4be873fbd28dec4583e7f737a045f546"
    python_version: str = "3.11"
    install_cmds: list = field(default_factory=lambda: ['pip install -e ".[dev]"'])
    min_testing: bool = True


@dataclass
class Dspy651a4c71(PythonProfile):
    owner: str = "stanfordnlp"
    repo: str = "dspy"
    commit: str = "651a4c715ecc6c5e68b68d22172768f0b20f2eea"


@dataclass
class Sympy2ab64612(PythonProfile):
    owner: str = "sympy"
    repo: str = "sympy"
    commit: str = "2ab64612efb287f09822419f4127878a4b664f71"
    min_testing: bool = True
    min_pregold: bool = True
    eval_sets: set[str] = field(
        default_factory=lambda: {
            "SWE-bench/SWE-bench",
            "SWE-bench/SWE-bench_Lite",
            "SWE-bench/SWE-bench_Verified",
        }
    )


@dataclass
class Tenacity0d40e76f(PythonProfile):
    owner: str = "jd"
    repo: str = "tenacity"
    commit: str = "0d40e76f7d06d631fb127e1ec58c8bd776e70d49"


@dataclass
class Termcolor3a42086f(PythonProfile):
    owner: str = "termcolor"
    repo: str = "termcolor"
    commit: str = "3a42086feb35647bc5aa5f1065b0327200da6b9b"


@dataclass
class TextdistanceC3aca916(PythonProfile):
    owner: str = "life4"
    repo: str = "textdistance"
    commit: str = "c3aca916bd756a8cb71114688b469ec90ef5b232"
    install_cmds: list = field(
        default_factory=lambda: ['pip install -e ".[benchmark,test]"']
    )


@dataclass
class TextfsmC31b6007(PythonProfile):
    owner: str = "google"
    repo: str = "textfsm"
    commit: str = "c31b600743895f018e7583f93405a3738a9f4d55"


@dataclass
class Thefuzz8a05a3ee(PythonProfile):
    owner: str = "seatgeek"
    repo: str = "thefuzz"
    commit: str = "8a05a3ee38cbd00a2d2f4bb31db34693b37a1fdd"


@dataclass
class Tinydb10644a0e(PythonProfile):
    owner: str = "msiemens"
    repo: str = "tinydb"
    commit: str = "10644a0e07ad180c5b756aba272ee6b0dbd12df8"


@dataclass
class Tldextract3d1bf184(PythonProfile):
    owner: str = "john-kurkowski"
    repo: str = "tldextract"
    commit: str = "3d1bf184d4f20fbdbadd6274560ccd438939160e"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[testing]"])


@dataclass
class Tomli443a0c1b(PythonProfile):
    owner: str = "hukkin"
    repo: str = "tomli"
    commit: str = "443a0c1bc5da39b7ed84306912ee1900e6b72e2f"


@dataclass
class TornadoD5ac65c1(PythonProfile):
    owner: str = "tornadoweb"
    repo: str = "tornado"
    commit: str = "d5ac65c1f1453c2aeddd089d8e68c159645c13e1"
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "python -m tornado.test --verbose"
    )

    def get_test_files(self, instance: dict) -> list[str]:
        f2p_files = set()
        p2p_files = set()
        for i, j in (
            (PASS_TO_PASS, p2p_files),
            (FAIL_TO_PASS, f2p_files),
        ):
            for test_name in instance[i]:
                is_match = re.search(r"\s\((.*)\)", test_name)
                if is_match:
                    test_path = is_match.group(1)
                    j.add("/".join(test_path.split(".")[:-1]) + ".py")
        return list(f2p_files), list(p2p_files)

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        for line in log.split("\n"):
            if line.endswith("... ok"):
                test_case = line.split(" ... ")[0]
                test_status_map[test_case] = TestStatus.PASSED.value
            elif " ... skipped " in line:
                test_case = line.split(" ... ")[0]
                test_status_map[test_case] = TestStatus.SKIPPED.value
            elif any([line.startswith(x) for x in ["ERROR:", "FAIL:"]]):
                test_case = " ".join(line.split()[1:3])
                test_status_map[test_case] = TestStatus.FAILED.value
        return test_status_map


@dataclass
class TrioCfbbe2c1(PythonProfile):
    owner: str = "python-trio"
    repo: str = "trio"
    commit: str = "cfbbe2c1f96e93b19bc2577d2cab3f4fe2e81153"


@dataclass
class Tweepy91a41c6e(PythonProfile):
    owner: str = "tweepy"
    repo: str = "tweepy"
    commit: str = "91a41c6e1c955d278c370d51d5cf43b05f7cd979"
    install_cmds: list = field(
        default_factory=lambda: ["pip install -e '.[dev,test,async]'"]
    )


@dataclass
class TypeguardB6a7e438(PythonProfile):
    owner: str = "agronholm"
    repo: str = "typeguard"
    commit: str = "b6a7e4387c30a9f7d635712157c889eb073c1ea3"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[test,doc]"])


@dataclass
class UsaddressA42a8f0c(PythonProfile):
    owner: str = "datamade"
    repo: str = "usaddress"
    commit: str = "a42a8f0c14bd2e273939fd51c604f10826301e73"
    install_cmds: list = field(default_factory=lambda: ["pip install -e .[dev]"])


@dataclass
class VoluptuousA7a55f83(PythonProfile):
    owner: str = "alecthomas"
    repo: str = "voluptuous"
    commit: str = "a7a55f83b9fa7ba68b0669b3d78a61de703e0a16"


@dataclass
class WebargsDbde72fe(PythonProfile):
    owner: str = "marshmallow-code"
    repo: str = "webargs"
    commit: str = "dbde72fe5db8a999acd1716d5ef855ab7cc1a274"


@dataclass
class WordCloudEc24191c(PythonProfile):
    owner: str = "amueller"
    repo: str = "word_cloud"
    commit: str = "ec24191c64570d287032c5a4179c38237cd94043"


@dataclass
class Xmltodict0952f382(PythonProfile):
    owner: str = "martinblech"
    repo: str = "xmltodict"
    commit: str = "0952f382c2340bc8b86a5503ba765a35a49cf7c4"


@dataclass
class Yamllint8513d9b9(PythonProfile):
    owner: str = "adrienverge"
    repo: str = "yamllint"
    commit: str = "8513d9b97da3b32453b3fccb221f4ab134a028d7"


@dataclass
class Moto694ce1f4(PythonProfile):
    owner: str = "getmoto"
    repo: str = "moto"
    commit: str = "694ce1f4880c784fed0553bc19b2ace6691bc109"
    python_version = "3.12"
    install_cmds: list = field(default_factory=lambda: ["make init"])
    min_testing: bool = True


@dataclass
class MypyE93f06ce(PythonProfile):
    owner: str = "python"
    repo: str = "mypy"
    commit: str = "e93f06ceab81d8ff1f777c7587d04c339cfd5a16"
    python_version = "3.12"
    install_cmds: list = field(
        default_factory=lambda: [
            "git submodule update --init mypy/typeshed || true",
            "python -m pip install -r test-requirements.txt",
            "python -m pip install -e .",
            "hash -r",
        ]
    )
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "pytest --color=no -rA -k"
    )
    min_testing: bool = True

    def get_test_cmd(self, instance: str, f2p_only: bool = False) -> tuple[str, list]:
        test_keys = []
        if f2p_only and FAIL_TO_PASS in instance:
            test_keys = [x.rsplit("::", 1)[-1] for x in instance[FAIL_TO_PASS]]
        elif INSTANCE_REF in instance and "test_patch" in instance[INSTANCE_REF]:
            test_keys = re.findall(
                r"\[case ([^\]]+)\]", instance[INSTANCE_REF]["test_patch"]
            )
        if len(test_keys) > 1:
            combined = " or ".join(test_keys)
            return f'{self.test_cmd} "{combined}"', test_keys
        return self.test_cmd, test_keys

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        for line in log.split("\n"):
            for status in [
                TestStatus.PASSED.value,
                TestStatus.FAILED.value,
            ]:
                if status in line:
                    test_case = line.split()[-1]
                    test_status_map[test_case] = status
                    break
        return test_status_map


@dataclass
class MONAIa09c1f08(PythonProfile):
    owner: str = "Project-MONAI"
    repo: str = "MONAI"
    commit: str = "a09c1f08461cec3d2131fde3939ef38c3c4ad5fc"
    python_version = "3.12"
    install_cmds: list = field(
        default_factory=lambda: [
            r"sed -i '/^git+https:\/\/github.com\/Project-MONAI\//d' requirements-dev.txt",
            "python -m pip install -U -r requirements-dev.txt",
            "python -m pip install -e .",
        ]
    )
    test_cmd: str = (
        "source /opt/miniconda3/bin/activate; "
        f"conda activate {ENV_NAME}; "
        "pytest --disable-warnings --color=no --tb=no --verbose"
    )
    min_pregold: bool = True
    min_testing: bool = True


@dataclass
class Dvc1d6ea681(PythonProfile):
    owner: str = "iterative"
    repo: str = "dvc"
    commit: str = "1d6ea68133289ceab2637ce7095772678af792c6"
    install_cmds: list = field(default_factory=lambda: ['pip install -e ".[dev]"'])
    min_testing: bool = True


@dataclass
class Hydra0f03eb60(PythonProfile):
    owner: str = "facebookresearch"
    repo: str = "hydra"
    commit: str = "0f03eb60c2ecd1fbdb25ede9a2c4faeac81de491"
    install_cmds: list = field(
        default_factory=lambda: [
            "apt-get update && apt-get install -y openjdk-17-jdk openjdk-17-jre",
            "pip install -e .",
        ]
    )


@dataclass
class Dask5f61e423(PythonProfile):
    owner: str = "dask"
    repo: str = "dask"
    commit: str = "5f61e42324c3a6cd4da17b5d5ebe4663aa4b8783"
    install_cmds: list = field(
        default_factory=lambda: [
            "python -m pip install graphviz",
            "python -m pip install -e .",
        ]
    )
    min_testing: bool = True


@dataclass
class Modin8c7799fd(PythonProfile):
    owner: str = "modin-project"
    repo: str = "modin"
    commit: str = "8c7799fdbbc2fb0543224160dd928215852b7757"
    install_cmds: list = field(default_factory=lambda: ['pip install -e ".[all]"'])
    min_pregold: bool = True
    min_testing: bool = True


@dataclass
class PydanticAcb0f10f(PythonProfile):
    owner: str = "pydantic"
    repo: str = "pydantic"
    commit: str = "acb0f10fda1c78441e052c57b4288bc91431f852"
    install_cmds: list = field(
        default_factory=lambda: [
            "apt-get update && apt-get install -y locales pipx",
            "pipx install uv",
            "pipx install pre-commit",
            'export PATH="$HOME/.local/bin:$PATH"',
            "make install",
        ]
    )
    test_cmd: str = (
        "/root/.local/bin/uv run pytest --disable-warnings --color=no --tb=no --verbose"
    )


@dataclass
class Conan86f29e13(PythonProfile):
    owner: str = "conan-io"
    repo: str = "conan"
    commit: str = "86f29e137a10bb6ed140c1a8c05c3099987b13c5"
    install_cmds = (
        INSTALL_CMAKE
        + INSTALL_BAZEL
        + [
            "apt-get -y update && apt-get -y upgrade && apt-get install -y build-essential cmake automake autoconf pkg-config meson ninja-build",
            "python -m pip install -r conans/requirements.txt",
            "python -m pip install -r conans/requirements_server.txt",
            "python -m pip install -r conans/requirements_dev.txt",
            "python -m pip install -e .",
        ]
    )
    min_testing: bool = True


@dataclass
class Pandas95280573(PythonProfile):
    owner: str = "pandas-dev"
    repo: str = "pandas"
    commit: str = "95280573e15be59036f98d82a8792599c10c6603"
    install_cmds: list = field(
        default_factory=lambda: [
            "git remote add upstream https://github.com/pandas-dev/pandas.git",
            "git fetch upstream --tags",
            "python -m pip install -ve . --no-build-isolation -Ceditable-verbose=true",
            """sed -i 's/__version__="[^"]*"/__version__="3.0.0.dev0+1992.g95280573e1"/' build/cp310/_version_meson.py""",
        ]
    )
    min_pregold: bool = True
    min_testing: bool = True


@dataclass
class MonkeyType70c3acf6(PythonProfile):
    owner: str = "Instagram"
    repo: str = "MonkeyType"
    commit: str = "70c3acf62950be5dfb28743c7a719bfdecebcd84"


@dataclass
class String2Stringc4a72f59(PythonProfile):
    owner: str = "stanfordnlp"
    repo: str = "string2string"
    commit: str = "c4a72f59aafe8db42c4015709078064535dc4191"
    install_cmds: list = field(
        default_factory=lambda: [
            "pip install -r docs/requirements.txt",
            "pip install -e .",
            "pip install pytest",
        ]
    )


# Register all Python profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, PythonProfile)
        and obj.__name__ != "PythonProfile"
    ):
        registry.register_profile(obj)



"""
This file is used to store common installation or testing patterns that may be
reused across different repositories / languages.
"""

CMAKE_VERSIONS = ["3.15.7", "3.16.9", "3.17.5", "3.19.7", "3.23.5", "3.27.9"]
INSTALL_CMAKE = (
    [
        f"wget https://github.com/Kitware/CMake/releases/download/v{v}/cmake-{v}-Linux-x86_64.tar.gz"
        for v in CMAKE_VERSIONS
    ]
    + [
        f"tar -xvzf cmake-{v}-Linux-x86_64.tar.gz && mv cmake-{v}-Linux-x86_64 /usr/share/cmake-{v}"
        if v not in ["3.23.5", "3.27.9"]
        else f"tar -xvzf cmake-{v}-Linux-x86_64.tar.gz && mv cmake-{v}-linux-x86_64 /usr/share/cmake-{v}"
        for v in CMAKE_VERSIONS
    ]
    + [
        f"update-alternatives --install /usr/bin/cmake cmake /usr/share/cmake-{v}/bin/cmake {(idx + 1) * 10}"
        for idx, v in enumerate(CMAKE_VERSIONS)
    ]
)

INSTALL_BAZEL = [
    cmd
    for v in ["6.5.0", "7.4.1", "8.0.0"]
    for cmd in [
        f"mkdir -p /usr/share/bazel-{v}/bin",
        f"wget https://github.com/bazelbuild/bazel/releases/download/{v}/bazel-{v}-linux-x86_64",
        f"chmod +x bazel-{v}-linux-x86_64",
        f"mv bazel-{v}-linux-x86_64 /usr/share/bazel-{v}/bin/bazel",
    ]
]

X11_DEPS = " ".join(
    [
        "libx11-xcb1",
        "libxcomposite1",
        "libxcursor1",
        "libxdamage1",
        "libxi6",
        "libxtst6",
        "libnss3",
        "libcups2",
        "libxss1",
        "libxrandr2",
        "libasound2",
        "libatk1.0-0",
        "libgtk-3-0",
        "x11-utils",
    ]
)



import re


PROMPT_KEYS = ["system", "demonstration", "instance"]


def extract_code_block(text: str) -> str:
    pattern = r"```(?:\w+)?\n(.*?)```"
    match = re.search(pattern, text, re.DOTALL)
    return match.group(1).strip() if match else ""



"""
Purpose: Automated construction of Docker images for repositories using profile registry.

Usage: python -m swesmith.build_repo.create_images --max-workers 4 -p django
"""

import argparse
import docker
import traceback
from collections import OrderedDict
from concurrent.futures import ThreadPoolExecutor, as_completed
from tqdm import tqdm

from swesmith.constants import Architecture
from swesmith.profiles import registry


def build_profile_image(profile, push=False):
    """
    Build a Docker image for a specific profile.

    Args:
        profile: A RepoProfile instance

    Returns:
        tuple: (profile_name, success: bool, error_message: str)
    """
    try:
        profile.create_mirror()
        profile.build_image()
        if push:
            profile.push_image()
        return (profile.image_name, True, None)
    except Exception as e:
        error_msg = f"Error building {profile.image_name}: {str(e)}"
        return (profile.image_name, False, error_msg)


def build_all_images(
    workers=4,
    repo_filter=None,
    proceed=False,
    push=False,
    force=False,
    arch=None,
):
    """
    Build Docker images for all registered profiles in parallel.

    Args:
        workers: Maximum number of parallel workers
        repo_filter: Optional list of repository name patterns to filter by (fuzzy matching)
        proceed: Whether to proceed without confirmation
        force: Force rebuild even if image already exists
        arch: Architecture to build for (e.g. "x86_64", "arm64")

    Returns:
        tuple: (successful_builds, failed_builds)
    """
    # Get all available profiles
    all_profiles = registry.values()

    # Update profile architecture if specified
    if arch:
        target_arch = arch
        print(f"Forcing build for architecture: {target_arch}")
        for profile in all_profiles:
            profile.arch = target_arch

    # Remove environments that have already been built
    client = docker.from_env()

    # Filter out profiles that already have images built (unless force is enabled)
    profiles_to_build = []
    if not force:
        for profile in all_profiles:
            try:
                # Check if image already exists
                client.images.get(profile.image_name)
            except docker.errors.ImageNotFound:
                profiles_to_build.append(profile)
    else:
        profiles_to_build = list(all_profiles)

    # Filter profiles if specified (fuzzy matching)
    if repo_filter:
        filtered_profiles = []
        for profile in profiles_to_build:
            # Check if any of the filter patterns appear in the image name
            if any(
                pattern.lower() in profile.image_name.lower() for pattern in repo_filter
            ):
                filtered_profiles.append(profile)
        profiles_to_build = filtered_profiles

    if not profiles_to_build:
        print("No profiles to build.")
        return [], []

    # Deduplicate profiles_to_build by image_name (more efficiently)
    profiles_to_build = list(
        OrderedDict(
            (profile.image_name, profile) for profile in profiles_to_build
        ).values()
    )

    print("Profiles to build:")
    for profile in sorted(profiles_to_build, key=lambda p: p.image_name):
        print(f"- {profile.image_name}")

    if not proceed:
        proceed = (
            input(
                f"Proceed with building {len(profiles_to_build)} images? (y/n): "
            ).lower()
            == "y"
        )
    if not proceed:
        return [], []

    # Build images in parallel
    successful, failed = [], []

    with tqdm(
        total=len(profiles_to_build), smoothing=0, desc="Building environment images"
    ) as pbar:
        with ThreadPoolExecutor(max_workers=workers) as executor:
            # Submit all build tasks
            future_to_profile = {
                executor.submit(build_profile_image, profile, push): profile
                for profile in profiles_to_build
            }

            # Process completed tasks
            for future in as_completed(future_to_profile):
                pbar.update(1)
                profile_name, success, error_msg = future.result()

                if success:
                    successful.append(profile_name)
                else:
                    failed.append(profile_name)
                    if error_msg:
                        print(f"\n{error_msg}")
                        traceback.print_exc()

    # Show results
    if len(failed) == 0:
        print("All environment images built successfully.")
    else:
        print(f"{len(failed)} environment images failed to build.")

    return successful, failed


def main():
    parser = argparse.ArgumentParser(
        description="Build Docker images for all registered repository profiles"
    )
    parser.add_argument(
        "-w",
        "--workers",
        type=int,
        default=4,
        help="Maximum number of parallel workers (default: 4)",
    )
    parser.add_argument(
        "-r",
        "--repos",
        type=str,
        nargs="+",
        help="Repository name patterns to build (fuzzy match, space-separated)",
    )
    parser.add_argument(
        "-y", "--proceed", action="store_true", help="Proceed without confirmation"
    )
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        help="Force rebuild even if image already exists",
    )
    parser.add_argument(
        "-p",
        "--push",
        action="store_true",
        help="Push built images to Docker Hub after building (default: False)",
    )
    parser.add_argument(
        "--list-envs", action="store_true", help="List all available profiles and exit"
    )
    parser.add_argument(
        "--arch",
        choices=[a.value for a in Architecture],
        help="Force build for specific architecture",
    )

    args = parser.parse_args()

    if args.list_envs:
        print("All execution environment Docker images:")
        for profile in registry.values():
            print(f"  {profile.image_name}")
        return

    successful, failed = build_all_images(
        workers=args.workers,
        repo_filter=args.repos,
        proceed=args.proceed,
        push=args.push,
        force=args.force,
        arch=args.arch,
    )

    if failed:
        print(f"- Failed builds: {failed}")
    if successful:
        print(f"- Successful builds: {len(successful)}")


if __name__ == "__main__":
    main()



"""
JavaScript remove modifiers for procedural bug generation using tree-sitter.
"""

import tree_sitter_javascript as tsjs
from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.javascript.base import JavaScriptProceduralModifier
from swesmith.constants import BugRewrite, CodeEntity, CodeProperty
from tree_sitter import Language, Parser

JS_LANGUAGE = Language(tsjs.language())


class RemoveLoopModifier(JavaScriptProceduralModifier):
    """Remove loop statements (for, while, do-while, for-in, for-of)"""

    explanation: str = CommonPMs.REMOVE_LOOP.explanation
    name: str = CommonPMs.REMOVE_LOOP.name
    conditions: list = CommonPMs.REMOVE_LOOP.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove a loop from the code."""
        # Parse the code
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        # Find and remove loop statements
        modified_code = self._remove_loops(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_loops(self, source_code: str, node) -> str:
        """Recursively find and remove loop statements."""
        removals = []

        def collect_loops(n):
            if n.type in [
                "for_statement",
                "for_in_statement",
                "for_of_statement",
                "while_statement",
                "do_statement",
            ]:
                if self.flip():
                    removals.append(n)
            for child in n.children:
                collect_loops(child)

        collect_loops(node)

        if not removals:
            return source_code

        # Apply removals from end to start to preserve byte offsets
        modified_source = source_code
        for loop_node in reversed(removals):
            start_byte = loop_node.start_byte
            end_byte = loop_node.end_byte

            # Remove the entire loop statement
            modified_source = modified_source[:start_byte] + modified_source[end_byte:]

        return modified_source


class RemoveConditionalModifier(JavaScriptProceduralModifier):
    """Remove conditional statements (if statements)"""

    explanation: str = CommonPMs.REMOVE_CONDITIONAL.explanation
    name: str = CommonPMs.REMOVE_CONDITIONAL.name
    conditions: list = CommonPMs.REMOVE_CONDITIONAL.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove an if statement from the code."""
        # Parse the code
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        # Find and remove conditional statements
        modified_code = self._remove_conditionals(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_conditionals(self, source_code: str, node) -> str:
        """Recursively find and remove if statements."""
        removals = []

        def collect_conditionals(n):
            if n.type == "if_statement":
                if self.flip():
                    removals.append(n)
            for child in n.children:
                collect_conditionals(child)

        collect_conditionals(node)

        if not removals:
            return source_code

        # Apply removals from end to start to preserve byte offsets
        modified_source = source_code
        for cond_node in reversed(removals):
            start_byte = cond_node.start_byte
            end_byte = cond_node.end_byte

            # Remove the entire conditional statement
            modified_source = modified_source[:start_byte] + modified_source[end_byte:]

        return modified_source


class RemoveAssignmentModifier(JavaScriptProceduralModifier):
    """Remove assignment statements"""

    explanation: str = CommonPMs.REMOVE_ASSIGNMENT.explanation
    name: str = CommonPMs.REMOVE_ASSIGNMENT.name
    conditions: list = CommonPMs.REMOVE_ASSIGNMENT.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove an assignment statement from the code."""
        # Parse the code
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        # Find and remove assignment statements
        modified_code = self._remove_assignments(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_assignments(self, source_code: str, node) -> str:
        """Recursively find and remove assignment statements."""
        removals = []

        def collect_assignments(n):
            # Look for assignment expressions and variable declarations
            if n.type in [
                "assignment_expression",
                "variable_declaration",
                "augmented_assignment_expression",
            ]:
                if self.flip():
                    # For expression statements, remove the whole statement including semicolon
                    if n.parent and n.parent.type == "expression_statement":
                        removals.append(n.parent)
                    else:
                        removals.append(n)
            for child in n.children:
                collect_assignments(child)

        collect_assignments(node)

        if not removals:
            return source_code

        # Apply removals from end to start to preserve byte offsets
        modified_source = source_code
        for assign_node in reversed(removals):
            start_byte = assign_node.start_byte
            end_byte = assign_node.end_byte

            # Find the end of the line (include semicolon and newline)
            while end_byte < len(modified_source) and modified_source[end_byte] in [
                " ",
                "\t",
                ";",
            ]:
                end_byte += 1
            if end_byte < len(modified_source) and modified_source[end_byte] == "\n":
                end_byte += 1

            # Remove the assignment statement
            modified_source = modified_source[:start_byte] + modified_source[end_byte:]

        return modified_source


class RemoveTernaryModifier(JavaScriptProceduralModifier):
    """Remove ternary expressions by replacing with just one branch."""

    explanation: str = "A ternary conditional expression may be missing - only one branch is being used."
    name: str = "func_pm_remove_ternary"
    conditions: list = [CodeProperty.IS_FUNCTION, CodeProperty.HAS_TERNARY]

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove ternary expressions by replacing with one of the branches."""
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._remove_ternary(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_ternary(self, source_code: str, node) -> str:
        """Find and remove ternary expressions by replacing with one branch."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        def collect_ternary_ops(n):
            if n.type == "ternary_expression" and len(n.children) >= 5:
                # Structure: condition ? consequent : alternative
                content_children = [c for c in n.children if c.type not in ["?", ":"]]
                if len(content_children) >= 3:
                    consequent = content_children[1]
                    alternative = content_children[2]

                    if self.flip():
                        # Randomly choose which branch to keep
                        keep_consequent = self.rand.choice([True, False])
                        changes.append(
                            {
                                "node": n,
                                "replacement": consequent
                                if keep_consequent
                                else alternative,
                            }
                        )

            for child in n.children:
                collect_ternary_ops(child)

        collect_ternary_ops(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            node = change["node"]
            replacement = change["replacement"]

            replacement_text = source_bytes[
                replacement.start_byte : replacement.end_byte
            ].decode("utf-8")

            modified_source = (
                modified_source[: node.start_byte]
                + replacement_text.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return modified_source.decode("utf-8")



"""
Given a folder of SWE-agent trajectories, extracts the trajectories
and transforms them into a fine-tuning compatible jsonl format, namely...

[
  {
    "messages": [
      {
        "role": "system",
        "content": "system prompt (optional)"
      },
      {
        "role": "user",
        "content": "human instruction"
      },
      {
        "role": "assistant",
        "content": "model response"
      }
    ]
  },
  ...
]

Usage: (from SWE-agent directory)
python -m swesmith.train.traj_mgr.collect_trajs --traj_dir <path> \
    --eval_dir <path> \
"""

import argparse
import json
import os
from pathlib import Path
from concurrent.futures import ThreadPoolExecutor, as_completed
from swebench.harness.constants import KEY_INSTANCE_ID, LOG_REPORT
from swesmith.constants import generate_hash
from swesmith.train.traj_mgr.utils import MAP_STYLE_TO_FUNC
from tqdm.auto import tqdm
from typing import Optional, Tuple


def process_single_trajectory(
    folder: str,
    traj_dir: Path,
    eval_dir: Path,
    transform_traj,
) -> Optional[Tuple[str, dict]]:
    """Process a single trajectory folder and return the result."""
    if not (eval_dir / folder).exists():
        return None
    if not (eval_dir / folder / LOG_REPORT).exists():
        return None

    try:
        report_path = eval_dir / folder / LOG_REPORT
        report = json.loads(report_path.read_text())
        is_resolved = (
            report.get("resolved", False)
            if folder not in report
            else report[folder].get("resolved", False)
        )

        pred_path = traj_dir / folder / f"{folder}.patch"
        traj_path = traj_dir / folder / f"{folder}.traj"
        traj_orig = json.loads(traj_path.read_text())
        traj = transform_traj(traj_orig)
        traj[KEY_INSTANCE_ID] = folder
        traj["resolved"] = is_resolved
        if "replay_config" in traj_orig:
            traj["model"] = json.loads(traj_orig["replay_config"])["agent"]["model"][
                "name"
            ]
        traj["traj_id"] = f"{folder}.{generate_hash(str(traj_dir))}"
        traj["patch"] = pred_path.read_text() if pred_path.exists() else ""

        return (folder, traj)
    except Exception as e:
        print(f"Error processing folder {folder}: {e}")
        return None


def main(
    out_dir: Path,
    traj_dir: Path,
    eval_dir: Path,
    style: str,
    workers: int,
):
    if style not in MAP_STYLE_TO_FUNC:
        raise ValueError(
            f"Style {style} not supported. Options: {list(MAP_STYLE_TO_FUNC.keys())}"
        )
    transform_traj = MAP_STYLE_TO_FUNC[style]

    folders = [x.name for x in traj_dir.iterdir() if x.is_dir()]
    print(f"Found {len(folders)} trajectory folders in {traj_dir}")

    out_path = out_dir / f"{eval_dir.name}.{style}.jsonl"

    # Process trajectories in parallel
    results = []
    with ThreadPoolExecutor(max_workers=workers) as executor:
        # Submit all tasks
        future_to_folder = {
            executor.submit(
                process_single_trajectory, folder, traj_dir, eval_dir, transform_traj
            ): folder
            for folder in folders
        }

        # Collect results as they complete
        for future in tqdm(
            as_completed(future_to_folder),
            total=len(folders),
            desc="Processing trajectories",
        ):
            result = future.result()
            if result is not None:
                results.append(result)

    # Write results to file
    num_trajs = 0
    with open(out_path, "w") as f:
        for _, traj in results:
            f.write(json.dumps(traj) + "\n")
            num_trajs += 1

    print(f"Wrote {num_trajs} valid trajectories to {out_path.absolute()}")


if __name__ == "__main__":
    user = os.getenv("USER")

    arg_parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    arg_parser.add_argument(
        "-t",
        "--traj_dir",
        type=Path,
        required=False,
        help="Path to folder containing SWE-agent trajectories. Default: trajectories/{user}/",
        default=f"trajectories/{user}/",
    )
    arg_parser.add_argument(
        "-e",
        "--eval_dir",
        type=Path,
        required=False,
        default="logs/run_evaluation/",
        help="Path to folder containing evaluation results. Default: logs/run_evaluation/",
    )
    arg_parser.add_argument(
        "-s",
        "--style",
        type=str,
        required=False,
        default="xml",
        choices=list(MAP_STYLE_TO_FUNC.keys()),
        help="Style of the trajectories",
    )
    arg_parser.add_argument(
        "-o",
        "--out_dir",
        type=Path,
        required=False,
        default=".",
        help="Path to output directory",
    )
    arg_parser.add_argument(
        "-w",
        "--workers",
        type=int,
        required=False,
        default=min(32, os.cpu_count() + 4),
        help="Maximum number of worker threads. Default: min(32, os.cpu_count() + 4)",
    )
    args = arg_parser.parse_args()
    main(**vars(args))



import re
import warnings

from swesmith.constants import CodeEntity, TODO_REWRITE
from tree_sitter import Language, Parser, Query, QueryCursor
import tree_sitter_c_sharp as tscs
from swesmith.bug_gen.adapters.utils import build_entity

C_SHARP_LANGUAGE = Language(tscs.language())


class CSharpEntity(CodeEntity):
    @property
    def name(self) -> str:
        name_query = Query(
            C_SHARP_LANGUAGE,
            """
                (constructor_declaration name: (identifier) @name)
                (destructor_declaration name: (identifier) @name)
                (method_declaration name: (identifier) @name)
            """,
        )
        name = self._extract_text_from_first_match(name_query, self.node, "name")
        if self.node.type == "destructor_declaration":
            name = f"{name} Finalizer"
        return name or ""

    @property
    def signature(self) -> str:
        body_query = Query(
            C_SHARP_LANGUAGE,
            """
            [
              (constructor_declaration body: (block) @body)
              (destructor_declaration body: (block) @body)
              (method_declaration body: (block) @body)
            ]
            """.strip(),
        )
        matches = QueryCursor(body_query).matches(self.node)
        if matches:
            body_node = matches[0][1]["body"][0]
            signature = (
                self.node.text[: body_node.start_byte - self.node.start_byte]
                .rstrip()
                .decode("utf-8")
            )
            signature = re.sub(r"\(\s+", "(", signature).strip()
            signature = re.sub(r"\s+\)", ")", signature).strip()
            signature = re.sub(r"\s+", " ", signature).strip()
            return signature
        return ""

    @property
    def stub(self) -> str:
        return f"{self.signature}\n{{\n\t// {TODO_REWRITE}\n}}"

    @staticmethod
    def _extract_text_from_first_match(query, node, capture_name: str) -> str | None:
        """Extract text from tree-sitter query matches with None fallback."""
        matches = QueryCursor(query).matches(node)
        return matches[0][1][capture_name][0].text.decode("utf-8") if matches else None


def get_entities_from_file_c_sharp(
    entities: list[CSharpEntity],
    file_path: str,
    max_entities: int = -1,
) -> None:
    """
    Parse a .cs file and return up to max_entities methods.
    If max_entities < 0, collects them all.
    """
    parser = Parser(C_SHARP_LANGUAGE)

    try:
        file_content = open(file_path, "r", encoding="utf8").read()
    except UnicodeDecodeError:
        warnings.warn(f"Ignoring file {file_path} as it has an unsupported encoding")
        return

    tree = parser.parse(bytes(file_content, "utf8"))
    root = tree.root_node
    lines = file_content.splitlines()

    def walk(node) -> None:
        # stop if we've hit the limit
        if 0 <= max_entities == len(entities):
            return

        if node.type == "ERROR":
            warnings.warn(f"Error encountered parsing {file_path}")
            return

        if node.type in [
            "constructor_declaration",
            "destructor_declaration",
            "method_declaration",
        ]:
            if node.type == "method_declaration" and not _has_body(node):
                pass
            else:
                entities.append(build_entity(node, lines, file_path, CSharpEntity))
                if 0 <= max_entities == len(entities):
                    return

        for child in node.children:
            walk(child)

    walk(root)


def _has_body(node) -> bool:
    """
    Check if a method declaration has a body.
    """
    for child in node.children:
        if child.type == "block":
            return True
    return False



from dataclasses import dataclass, field

from swesmith.constants import ENV_NAME
from swebench.harness.constants import TestStatus
from swesmith.profiles.base import RepoProfile, registry


@dataclass
class RustProfile(RepoProfile):
    """
    Profile for Rust repositories.
    """

    test_cmd: str = "cargo test --verbose"
    exts: list[str] = field(default_factory=lambda: [".rs"])
    rust_version: str = "1.88"

    def log_parser(self, log: str):
        test_status_map = {}
        for line in log.splitlines():
            line = line.removeprefix("test ")
            if "... ok" in line:
                test_name = line.rsplit(" ... ", 1)[0].strip()
                test_status_map[test_name] = TestStatus.PASSED.value
            elif "... FAILED" in line:
                test_name = line.rsplit(" ... ", 1)[0].strip()
                test_status_map[test_name] = TestStatus.FAILED.value
        return test_status_map

    @property
    def dockerfile(self):
        return f"""FROM rust:{self.rust_version}
ARG DEBIAN_FRONTEND=noninteractive
ENV TZ=Etc/UTC

RUN apt update && apt install -y wget git build-essential \
&& rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN {self.test_cmd} || true
"""


@dataclass
class Base64cac5ff84(RustProfile):
    owner: str = "marshallpierce"
    repo: str = "rust-base64"
    commit: str = "cac5ff84cd771b1a9f52da020b053b35f0ff3ede"


@dataclass
class Clap3716f9f4(RustProfile):
    owner: str = "clap-rs"
    repo: str = "clap"
    commit: str = "3716f9f4289594b43abec42b2538efd1a90ff897"
    test_cmd: str = "make test-full ARGS=--verbose"


@dataclass
class Hyperc88df788(RustProfile):
    owner: str = "hyperium"
    repo: str = "hyper"
    commit: str = "c88df7886c74a1ade69c0b4c68eaf570c8111622"
    test_cmd: str = "cargo test --verbose --features full"


@dataclass
class Itertools041c733c(RustProfile):
    owner: str = "rust-itertools"
    repo: str = "itertools"
    commit: str = "041c733cb6fbfe6aae5cce28766dc6020043a7f9"
    test_cmd: str = "cargo test --verbose --all-features"


@dataclass
class Jsoncd55b5a0(RustProfile):
    owner: str = "serde-rs"
    repo: str = "json"
    commit: str = "cd55b5a0ff5f88f1aeb7a77c1befc9ddb3205201"


@dataclass
class Log3aa1359e(RustProfile):
    owner: str = "rust-lang"
    repo: str = "log"
    commit: str = "3aa1359e926a39f841791207d6e57e00da3e68e2"


@dataclass
class Semver37bcbe69(RustProfile):
    owner: str = "dtolnay"
    repo: str = "semver"
    commit: str = "37bcbe69d9259e4770643b63104798f7cc5d653c"


@dataclass
class Tokioab3ff69c(RustProfile):
    owner: str = "tokio-rs"
    repo: str = "tokio"
    commit: str = "ab3ff69cf2258a8c696b2dca89a2cef4ff114c1c"
    test_cmd: str = "cargo test --verbose --features full -- --skip try_exists"
    timeout: int = 180
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )


@dataclass
class Uuid2fd9b614(RustProfile):
    owner: str = "uuid-rs"
    repo: str = "uuid"
    commit: str = "2fd9b614c92e4e4b18928e2f539d82accf8eaeee"
    test_cmd: str = "cargo test --verbose --all-features"


@dataclass
class MdBook37273ba8(RustProfile):
    owner: str = "rust-lang"
    repo: str = "mdBook"
    commit: str = "37273ba8e0f86771b02f3a8a4bd3b0b3d388c573"
    test_cmd: str = "cargo test --workspace --verbose"


@dataclass
class RustCSVda000888(RustProfile):
    owner: str = "BurntSushi"
    repo: str = "rust-csv"
    commit: str = "da0008884062cf222ceb9c05f006be4bb6ac38a7"


@dataclass
class Html5everb93afc94(RustProfile):
    owner: str = "servo"
    repo: str = "html5ever"
    commit: str = "b93afc9484cf5de40b422a44f9cea86ab371e3ee"

    @property
    def dockerfile(self):
        return f"""FROM rust:1.88
ARG DEBIAN_FRONTEND=noninteractive
ENV TZ=Etc/UTC

RUN apt update && apt install -y wget git build-essential \
&& rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN git submodule update --init
"""


@dataclass
class Byteorder5a82625f(RustProfile):
    owner: str = "BurntSushi"
    repo: str = "byteorder"
    commit: str = "5a82625fae462e8ba64cec8146b24a372b4d75c6"


@dataclass
class Chronod43108cb(RustProfile):
    owner: str = "chronotope"
    repo: str = "chrono"
    commit: str = "d43108cbfc884b0864d1cf2db7719aedf4adbf23"


@dataclass
class Rpds3e7c8ae6(RustProfile):
    owner: str = "orium"
    repo: str = "rpds"
    commit: str = "3e7c8ae693cdc6e1b255c87279b6ad8aded6401d"


@dataclass
class Ripgrep3b7fd442(RustProfile):
    owner: str = "BurntSushi"
    repo: str = "ripgrep"
    commit: str = "3b7fd442a6f3aa73f650e763d7cbb902c03d700e"
    test_cmd: str = "cargo test --all --verbose"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return f"""FROM rust:1.88
ARG DEBIAN_FRONTEND=noninteractive
ENV TZ=Etc/UTC

RUN apt update && apt install -y wget git build-essential \
&& rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN cargo build --release
"""


@dataclass
class RustClippyf4f579f4(RustProfile):
    owner: str = "rust-lang"
    repo: str = "rust-clippy"
    commit: str = "f4f579f4ac455b76ddadc85553ba19b115dd144e"


@dataclass
class Hexyl2e264378(RustProfile):
    owner: str = "sharkdp"
    repo: str = "hexyl"
    commit: str = "2e2643782d6ced9b5ac75596169a79127d8e535a"


@dataclass
class Oha8dc63499(RustProfile):
    owner: str = "hatoo"
    repo: str = "oha"
    commit: str = "8dc63499f84b3116652987dac711eca687ccc2fd"


@dataclass
class Indicatifdbd26eb18(RustProfile):
    owner: str = "console-rs"
    repo: str = "indicatif"
    commit: str = "dbd26eb18157e5fad18c79e1933ad5f249165d6c"


@dataclass
class Melodyf4af9b48(RustProfile):
    owner: str = "yoav-lavi"
    repo: str = "melody"
    commit: str = "f4af9b4829555fedb687eb5f6f5755ce3c5ef738"


@dataclass
class RustOwl655bc5c3(RustProfile):
    owner: str = "cordx56"
    repo: str = "rust-owl"
    commit: str = "655bc5c37e59156954fa9af3e6466602e7dfa814"


@dataclass
class Quinnbb359ccd(RustProfile):
    owner: str = "quinn-rs"
    repo: str = "quinn"
    commit: str = "bb359ccd7dfbc18b472bcb61e6800be6dc886264"


@dataclass
class Shellharden6a6ffd42(RustProfile):
    owner: str = "anordal"
    repo: str = "shellharden"
    commit: str = "6a6ffd42f6a9d8b558d479346d09233ae5e7a2ae"


@dataclass
class Grexfa3e8ed7(RustProfile):
    owner: str = "pemistahl"
    repo: str = "grex"
    commit: str = "fa3e8ed71c43ea92f9f0e43ab31e1c82d006d5dd"


@dataclass
class Htmlq6e31bc81(RustProfile):
    owner: str = "mgdm"
    repo: str = "htmlq"
    commit: str = "6e31bc814332b2521f0316d0ed9bf0a1c521b6e6"


@dataclass
class Xh4a6e44fc(RustProfile):
    owner: str = "ducaale"
    repo: str = "xh"
    commit: str = "4a6e44fcb562126959c54ca33b673cf7d707a63e"


@dataclass
class Lightningcss400f705e(RustProfile):
    owner: str = "parcel-bundler"
    repo: str = "lightningcss"
    commit: str = "400f705e63e139c326f480aed11e1416f5a3a61f"


@dataclass
class Miniserve8449e8b1(RustProfile):
    owner: str = "svenstaro"
    repo: str = "miniserve"
    commit: str = "8449e8b118ffd61de8df970b56b0796f891e3697"


@dataclass
class Tailpsin6278437c(RustProfile):
    owner: str = "bensadeh"
    repo: str = "tailpsin"
    commit: str = "6278437c3d28f2e95201f3b0c8a471b8668eed5b"


@dataclass
class SccacheCd7dcd5f(RustProfile):
    owner: str = "mozilla"
    repo: str = "sccache"
    commit: str = "cd7dcd5f73b7c77b826ad5173b6af6642ad03a3e"


@dataclass
class Boa14e5c634(RustProfile):
    owner: str = "boa-dev"
    repo: str = "boa"
    commit: str = "14e5c6342d72ef128ecad92f66f4c54641bd9561"


@dataclass
class Pastelb60e8993(RustProfile):
    owner: str = "sharkdp"
    repo: str = "pastel"
    commit: str = "b60e89932629b2d162e1ff9f3976a5a0ef1e5db9"


@dataclass
class Anyhow2c0bda4c(RustProfile):
    owner: str = "dtolnay"
    repo: str = "anyhow"
    commit: str = "2c0bda4ce944d943e7141f0316b0ea996602238e"


@dataclass
class Cxx0d80b351(RustProfile):
    owner: str = "dtolnay"
    repo: str = "cxx"
    commit: str = "0d80b351886a00af9a7120369f22a0b7f0affd72"


@dataclass
class Rustfmt86261bfb(RustProfile):
    owner: str = "rust-lang"
    repo: str = "rustfmt"
    commit: str = "86261bfb87a207030b1dfeef0f832ac13f369b1a"


@dataclass
class TealdeerC5d62e59(RustProfile):
    owner: str = "tealdeer-rs"
    repo: str = "tealdeer"
    commit: str = "c5d62e5987b38705814b72354373c50fe165dbb3"


@dataclass
class Image26edc698(RustProfile):
    owner: str = "image-rs"
    repo: str = "image"
    commit: str = "26edc698463cc2a0e7b9735ca41d375dce0449a2"


@dataclass
class Duacli8570c154(RustProfile):
    owner: str = "Byron"
    repo: str = "dua-cli"
    commit: str = "8570c1543e3cd0983725f6e1938bf3e73442678a"


@dataclass
class Serenityc6219206(RustProfile):
    owner: str = "serenity-rs"
    repo: str = "serenity"
    commit: str = "c6219206a38161a9e8d78660f19b44ba4dfb4ed9"


@dataclass
class Tideb32f680d(RustProfile):
    owner: str = "http-rs"
    repo: str = "tide"
    commit: str = "b32f680d5bd14bc2ce7c81bef9ce99859028b20f"


@dataclass
class Rhai6b132e55(RustProfile):
    owner: str = "rhaiscript"
    repo: str = "rhai"
    commit: str = "6b132e55167e6fc82a2348e90d507141e1204a12"


@dataclass
class Rayon5b4eb339(RustProfile):
    owner: str = "rayon-rs"
    repo: str = "rayon"
    commit: str = "5b4eb339c06943cbb71d8368e78343c049e6d71c"


@dataclass
class Brootd6c798ed(RustProfile):
    owner: str = "Canop"
    repo: str = "broot"
    commit: str = "d6c798edbd136dbe5b67566ad74a7daa97e8ae49"


@dataclass
class OneFetchE5958cec(RustProfile):
    owner: str = "o2sh"
    repo: str = "onefetch"
    commit: str = "e5958cec1e5d17d72405f1c96cf30ae1e2defa16"


@dataclass
class Reqwest01f03a4c(RustProfile):
    owner: str = "seanmonstar"
    repo: str = "reqwest"
    commit: str = "01f03a4c01fb13e2262a513ed21e2b84b5186f46"


@dataclass
class Dust62bf1e14(RustProfile):
    owner: str = "bootandy"
    repo: str = "dust"
    commit: str = "62bf1e14de73b14bdf5c691be29e6dcc4de352aa"


@dataclass
class Bore8e059cda(RustProfile):
    owner: str = "ekzhang"
    repo: str = "bore"
    commit: str = "8e059cdaf993d25d92080a2b28a71949a4545d03"


@dataclass
class Warp3449d3d9(RustProfile):
    owner: str = "seanmonstar"
    repo: str = "warp"
    commit: str = "3449d3d9816ea3898059e62b5325716c1cc27c8b"


@dataclass
class Gping26eb5b91(RustProfile):
    owner: str = "orf"
    repo: str = "gping"
    commit: str = "26eb5b914b1d90d75ebf23c3a9ae8ee3ebd6f217"


@dataclass
class TokenizersEcad3f18(RustProfile):
    owner: str = "huggingface"
    repo: str = "tokenizers"
    commit: str = "ecad3f18a3e340635f5393cfb22cf70d3502f64a"
    test_cmd: str = f"cd ~/{ENV_NAME}/tokenizers && cargo test --verbose"

    @property
    def dockerfile(self):
        return f"""FROM rust:{self.rust_version}
ARG DEBIAN_FRONTEND=noninteractive
ENV TZ=Etc/UTC

RUN apt update && apt install -y wget git build-essential \
&& rm -rf /var/lib/apt/lists/*

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN {self.test_cmd} || true
"""


# Register all Rust profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, RustProfile)
        and obj.__name__ != "RustProfile"
    ):
        registry.register_profile(obj)



import libcst

from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.python.base import PythonProceduralModifier


class ClassRemoveBasesModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.CLASS_REMOVE_BASES.explanation
    name: str = CommonPMs.CLASS_REMOVE_BASES.name
    conditions: list = CommonPMs.CLASS_REMOVE_BASES.conditions
    min_complexity: int = 10

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_ClassDef(self, original_node, updated_node):
            bases = list(updated_node.bases)
            if len(bases) > 0 and self.flip():
                if len(bases) == 1:
                    bases = []
                else:
                    to_remove = self.parent.rand.randint(0, len(bases) - 1)
                    bases.pop(to_remove)
            return updated_node.with_changes(bases=tuple(bases))


class ClassShuffleMethodsModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.CLASS_SHUFFLE_METHODS.explanation
    name: str = CommonPMs.CLASS_SHUFFLE_METHODS.name
    conditions: list = CommonPMs.CLASS_SHUFFLE_METHODS.conditions
    min_complexity: int = 10

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_ClassDef(self, original_node, updated_node):
            methods = [
                n for n in updated_node.body.body if isinstance(n, libcst.FunctionDef)
            ]
            non_methods = [
                n
                for n in updated_node.body.body
                if not isinstance(n, libcst.FunctionDef)
            ]
            self.parent.rand.shuffle(methods)
            new_body = non_methods + methods
            return updated_node.with_changes(
                body=updated_node.body.with_changes(body=tuple(new_body))
            )


class ClassRemoveFuncsModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.CLASS_REMOVE_FUNCS.explanation
    name: str = CommonPMs.CLASS_REMOVE_FUNCS.name
    conditions: list = CommonPMs.CLASS_REMOVE_FUNCS.conditions
    min_complexity: int = 10

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_ClassDef(
            self, original_node: libcst.ClassDef, updated_node: libcst.ClassDef
        ) -> libcst.ClassDef:
            # Access the statements inside the indented block
            body_statements = list(updated_node.body.body)

            # Track which function names we're removing
            removed_functions = set()

            # First pass: identify functions to remove
            new_body_statements = []
            for stmt in body_statements:
                if isinstance(stmt, libcst.FunctionDef) and self.flip():
                    # Track this function name for removal
                    removed_functions.add(stmt.name.value)
                    # Skip this function (remove it)
                    continue
                new_body_statements.append(stmt)

            # Only proceed if we actually removed something
            if not removed_functions:
                return updated_node

            # Create a reference remover to clean up references to removed functions
            reference_remover = FunctionReferenceRemover(removed_functions)

            # Second pass: process the remaining statements to remove references
            clean_statements = []
            for stmt in new_body_statements:
                # The correct way to apply a transformer to a node
                clean_stmt = stmt.visit(reference_remover)
                clean_statements.append(clean_stmt)

            # Create a new indented block with the cleaned statements
            new_body = updated_node.body.with_changes(body=tuple(clean_statements))

            # Return the updated class with the new body
            return updated_node.with_changes(body=new_body)


class FunctionReferenceRemover(libcst.CSTTransformer):
    """Helper transformer to remove references to deleted functions."""

    def __init__(self, removed_functions):
        super().__init__()
        self.removed_functions = removed_functions
        self.in_self_attr = False

    def visit_Attribute(self, node: libcst.Attribute) -> bool:
        # Check if this is a self.method_name pattern
        if (
            isinstance(node.value, libcst.Name)
            and node.value.value == "self"
            and node.attr.value in self.removed_functions
        ):
            self.in_self_attr = True
        return True

    def leave_Attribute(
        self, original_node: libcst.Attribute, updated_node: libcst.Attribute
    ) -> libcst.BaseExpression:
        if (
            isinstance(updated_node.value, libcst.Name)
            and updated_node.value.value == "self"
            and updated_node.attr.value in self.removed_functions
        ):
            # Reset state
            self.in_self_attr = False
        return updated_node

    def leave_Call(
        self, original_node: libcst.Call, updated_node: libcst.Call
    ) -> libcst.BaseExpression:
        # Check if we're calling a removed function through self
        if self.in_self_attr:
            # Reset state
            self.in_self_attr = False
            # Replace with a placeholder that won't cause errors
            return libcst.Name(value="None")
        return updated_node



"""
Purpose: Combine multiple patches from the same module into a single patch.

Usage: python swesmith/bug_gen/combine/same_module.py \
    --bug_gen_dir <path to bug_gen dir> \
"""

import argparse
import json
import os
import re
import subprocess

from pathlib import Path
from swebench.harness.constants import KEY_INSTANCE_ID
from swesmith.constants import (
    LOG_DIR_BUG_GEN,
    LOG_DIR_TASKS,
    PREFIX_BUG,
    PREFIX_METADATA,
    generate_hash,
)
from swesmith.bug_gen.utils import apply_patches, get_combos
from swesmith.profiles import registry
from tqdm.auto import tqdm
from unidiff import PatchSet

COMBINE_MODULE = "combine_module"
EXCLUDED_BUG_TYPES = ["func_basic", "combine_file", "combine_module", "pr_mirror"]


def convert_to_path(folder: str):
    DUNDER_PATTERN = r"____[a-zA-z\d]+__\.py$"
    path = "/".join(folder.split("__")[2:])  # Exclude <repo>__<commit>
    if re.search(DUNDER_PATTERN, folder):
        path = path.replace("//", "/__").replace("/.py", "__.py")
    return path


def get_patches_from_folder(folder_path, include_patches=None):
    """Get all patch file paths from a folder."""
    if include_patches is None:
        include_patches = []
    patch_files = []
    for root, _, files in os.walk(folder_path):
        for file in files:
            if not file.endswith(".diff"):
                continue
            if len(include_patches) > 0 and file not in include_patches:
                continue
            patch_files.append(os.path.join(root, file))
    return patch_files


def remove_paths(current, depth, bug_gen_dir):
    """Remove all paths that are less than depth."""
    if depth == 0:
        return
    keys = list(current.keys())
    for k in keys:
        if isinstance(current[k], dict):
            remove_paths(current[k], depth - 1, bug_gen_dir)
            if len(current[k]) == 0:
                del current[k]
        else:
            current[k] = get_patches_from_folder(bug_gen_dir / k)


def remove_empty_paths(current):
    """Remove all empty paths."""
    keys = list(current.keys())
    for k in keys:
        if isinstance(current[k], dict):
            remove_empty_paths(current[k])
            if len(current[k]) == 0:
                del current[k]
        else:
            if len(current[k]) == 0:
                del current[k]


def collapse_subdicts(current, depth, prefix=""):
    """
    Collapse keys up to a certain depth.
    - If i have {"foo": {"bar": {"baz": 1}}} and depth = 2
    => I want to get {"foo_bar": {"baz": 1}}
    - If i have {"foo": {"bar": {"baz": 1}}} and depth of 3
    => I want to get {"foo_bar_baz": 1}
    - If i have {"foo": {"bar": {"baz": 1, "blow": 2}}} and depth of 3
    => I want to get {"foo_bar_baz": 1, "foo_bar_blow": 2}
    """
    if depth == 0 or not isinstance(current, dict):
        return current
    new_dict = {}
    for k, v in current.items():
        new_key = prefix + k
        if isinstance(v, dict) and depth > 1:
            collapsed = collapse_subdicts(v, depth - 1, new_key + "/")
            if isinstance(collapsed, dict):
                new_dict.update(collapsed)
            else:
                new_dict[new_key] = collapsed
        else:
            new_dict[new_key] = v
    return new_dict


def convert_nested_dict_to_list(nested_dict):
    """Convert a nested dict to a list of values."""
    result = []
    for value in nested_dict.values():
        value = convert_nested_dict_to_list(value) if isinstance(value, dict) else value
        result.extend(value)
    return result


def main(
    bug_gen_dir: str,
    depth: int,
    num_patches: int,
    limit_per_module: int,
    max_combos: int,
    include_invalid_patches: bool = False,
):
    assert bug_gen_dir.startswith(str(LOG_DIR_BUG_GEN)), (
        f"bug_gen_dir must be of form {str(LOG_DIR_BUG_GEN)}/<repo name>"
    )
    repo = bug_gen_dir.strip("/").split("/")[-1]
    bug_gen_dir = Path(bug_gen_dir)
    folders = [
        x
        for x in os.listdir(bug_gen_dir)
        if x not in EXCLUDED_BUG_TYPES and os.path.isdir(bug_gen_dir / x)
    ]

    print(f"[{repo}] Extracting patch groups at depth {depth}")

    # Construct map_path_to_patches[path][to][file] = [patches]
    map_path_to_patches = {}
    for folder in folders:
        path = convert_to_path(folder).split("/")
        current = map_path_to_patches
        for p in path:
            if p not in current:
                if p.endswith(".py"):
                    current[p] = []
                    break
                current[p] = {}
            current = current[p]
        current[p] = get_patches_from_folder(bug_gen_dir / folder)

    # Get validated instance ids
    validated_inst_ids = []
    if not include_invalid_patches:
        validated_inst_ids = [
            x[KEY_INSTANCE_ID].split(".")[-1]
            for x in json.load(open(os.path.join(LOG_DIR_TASKS, f"{repo}.json")))
        ]

    # Given map_patch_to_patches[path][to][file] = [patches]...
    # * Remove all paths < depth
    # * Collapse all remaining subdicts into a single dict
    remove_paths(map_path_to_patches, depth, bug_gen_dir)
    remove_empty_paths(map_path_to_patches)
    map_path_to_patches = collapse_subdicts(map_path_to_patches, depth)
    for k in list(map_path_to_patches.keys()):
        map_path_to_patches[k] = [
            x
            for x in convert_nested_dict_to_list(map_path_to_patches[k])
            if not include_invalid_patches
            and x.split(f"{PREFIX_BUG}__")[-1].split(".diff")[0] in validated_inst_ids
        ]
        if k.endswith(".py") or len(map_path_to_patches[k]) == 0:
            del map_path_to_patches[k]

    if map_path_to_patches == {}:
        print(f"[{repo}] No modules at file depth {depth} with multiple patches found")
        return
    print(
        f"[{repo}] Found {len(map_path_to_patches)} modules at file depth {depth} with multiple patches"
    )

    # For each module
    registry.get(repo).clone()
    total_success, total_fails = 0, 0
    for path, patches in tqdm(map_path_to_patches.items()):
        combos = get_combos(patches, num_patches, max_combos)
        i, success = 0, 0
        while i < len(combos):
            combo = combos[i]
            patch = apply_patches(repo, combo)
            if patch is not None and len(PatchSet(patch)) > 1:
                success += 1
                file_name = f"{COMBINE_MODULE}__{generate_hash(patch)}"
                file_parent = bug_gen_dir / COMBINE_MODULE
                file_parent.mkdir(parents=True, exist_ok=True)
                with open(file_parent / f"{PREFIX_BUG}__{file_name}.diff", "w") as f:
                    f.write(patch)
                with open(
                    file_parent / f"{PREFIX_METADATA}__{file_name}.json", "w"
                ) as f:
                    json.dump(
                        {
                            "patch_files": [
                                Path(f).name.rsplit(".", 1)[0] for f in combo
                            ],
                            "num_patch_files": len(combo),
                        },
                        f,
                        indent=4,
                    )
                if limit_per_module != -1 and success >= limit_per_module:
                    break
                # Regenerate combos from unused patches
                patches = [p for p in patches if p not in set(combo)]
                combos = get_combos(patches, num_patches, max_combos)
                i = 0
            else:
                total_fails += 1
                i += 1
        total_success += success

    print(f"{repo}: {total_success} successes, {total_fails} fails")
    subprocess.run(["rm", "-rf", repo], check=True)


if __name__ == "__main__":
    parser = argparse.ArgumentParser("Combine patches from the same module")
    parser.add_argument("bug_gen_dir", type=str, help="Path to the bug_gen dir")
    parser.add_argument(
        "--num_patches",
        type=int,
        help="Number of patches to merge.",
        default=2,
    )
    parser.add_argument(
        "--limit_per_module",
        type=int,
        help="Maximum number of merged patches to keep per file (default no limit).",
        default=-1,
    )
    parser.add_argument(
        "--max_combos",
        type=int,
        help="Maximum number of combinations to try (-1 for no limit).",
        default=100,
    )
    parser.add_argument(
        "--include_invalid_patches", action="store_true", help="Include invalid patches"
    )
    parser.add_argument(
        "--depth",
        type=int,
        help="Depth of the module to combine patches from",
        default=3,
    )
    args = parser.parse_args()
    main(**vars(args))



"""
Purpose: Given a task instance, use the execution output of running tests as issue text.

python swesmith/issue_gen/get_from_tests.py logs/experiments/*.json \
    --config_file configs/issue_gen/ig_tests.yaml \
    --model anthropic/claude-3-7-sonnet-20250219 \
    --n_workers 1
"""

import argparse
import docker
import litellm
import json
import random
import subprocess
import yaml

from concurrent.futures import ThreadPoolExecutor, as_completed
from datasets import load_dataset
from litellm import completion, completion_cost
from pathlib import Path
from swebench.harness.constants import (
    DOCKER_USER,
    DOCKER_WORKDIR,
    FAIL_TO_PASS,
    KEY_INSTANCE_ID,
)
from swebench.harness.docker_utils import (
    cleanup_container,
    copy_to_container,
    exec_run_with_timeout,
)
from swesmith.constants import (
    KEY_IMAGE_NAME,
    LOG_DIR_ISSUE_GEN,
    TEST_OUTPUT_END,
    TEST_OUTPUT_START,
)
from swesmith.issue_gen.utils import get_test_function
from swesmith.profiles import RepoProfile, registry
from swesmith.profiles.python import PythonProfile
from tqdm.auto import tqdm
from tqdm.contrib.logging import logging_redirect_tqdm

litellm.drop_params = True
litellm.suppress_debug_info = True

LOG_FILE_ISSUE = "issue_test.txt"
LOG_FILE_METADATA = "metadata_test.json"
SWEBV_PS = load_dataset("SWE-bench/SWE-bench_Verified", split="test")[
    "problem_statement"
]
TEST_INFO = """**Test Source Code**
```python
{func}
```

**Test Output**
```
{output}
```
"""


def get_verbose_test_cmd(instance: dict, rp: RepoProfile, test_idx: int | None = None):
    """
    Get test command that runs a random F2P test verbosely.
    """
    test_cmd = rp.test_cmd
    # TODO: This should probably be changed, or incorporated into the profile
    if test_cmd == PythonProfile.test_cmd:
        test_cmd = test_cmd.replace(
            PythonProfile.test_cmd, "pytest -v --showlocals --tb=long --color=no"
        )
    f2p_test = (
        random.choice(instance[FAIL_TO_PASS])
        if test_idx is None
        else instance[FAIL_TO_PASS][test_idx]
    )
    test_cmd += " " + f2p_test
    return test_cmd


def run_command_in_container(instance: dict, command: str, rp: RepoProfile):
    """
    Run a command in a docker container.
    """
    container = None
    client = docker.from_env()
    instance_id = instance[KEY_INSTANCE_ID]
    image_name = instance[KEY_IMAGE_NAME]

    # Start docker container
    container_name = f"swesmith.inst_gen.{instance_id}"
    container = client.containers.create(
        image=image_name,
        name=container_name,
        user=DOCKER_USER,
        detach=True,
        command="tail -f /dev/null",
        platform="linux/x86_64",
        mem_limit="10g",
    )
    container.start()

    # Set up command run script
    eval_file = Path(f"eval_{instance_id}.sh")
    eval_file.write_text(
        "\n".join(
            [
                "#!/bin/bash",
                "set -uxo pipefail",
                f"cd {DOCKER_WORKDIR}",
                "git fetch",
                f"git checkout {instance[KEY_INSTANCE_ID]}",
                f": '{TEST_OUTPUT_START}'",
                command,
                f": '{TEST_OUTPUT_END}'",
            ]
        )
    )
    copy_to_container(container, eval_file, Path("/eval.sh"))
    eval_file.unlink()

    # Checkout the commit corresponding to the bug + run testing command
    container.exec_run("git fetch", workdir=DOCKER_WORKDIR, user=DOCKER_USER)
    container.exec_run(
        f"git checkout {instance['instance_id']}",
        workdir=DOCKER_WORKDIR,
        user=DOCKER_USER,
    )
    test_output, _, _ = exec_run_with_timeout(
        container, "/bin/bash /eval.sh", timeout=rp.timeout
    )
    start_idx = test_output.find(TEST_OUTPUT_START) + len(TEST_OUTPUT_START)
    end_idx = test_output.find(TEST_OUTPUT_END)
    lines = [
        x
        for x in test_output[start_idx:end_idx].splitlines()
        if x.strip() != command.strip()
    ]
    test_output = "\n".join(lines[1:-1])
    cleanup_container(client, container, "quiet")
    return test_output


def _process_instance(instance: dict, config_file: str | None, model: str | None):
    log_dir = (
        LOG_DIR_ISSUE_GEN
        / "from_tests"
        / instance["repo"].split("/")[-1]
        / instance[KEY_INSTANCE_ID]
    )
    path_metadata = log_dir / LOG_FILE_METADATA
    path_issue = log_dir / LOG_FILE_ISSUE

    cloned = False
    if log_dir.exists() and path_metadata.exists() and path_issue.exists():
        with open(path_metadata, "r") as f:
            metadata = json.load(f)
        test_idx = metadata["test_idx"]
        test_info = metadata["test_info"]
        test_output = metadata["test_output"]
        test_src = metadata["test_src"]

        if (
            metadata["generated"]
            or not metadata["generated"]
            and not config_file
            and not model
        ):
            return {"completed": 1, "timed_out": 0, "failed": 0}
    else:
        test_idx = random.randint(0, len(instance[FAIL_TO_PASS]) - 1)
        rp = registry.get_from_inst(instance)
        cmd = get_verbose_test_cmd(instance, rp, test_idx)
        test_output = run_command_in_container(instance, cmd, rp)
        test_func = get_test_function(instance, test_idx)
        test_src = test_func["test_src"]
        test_info = TEST_INFO.format(func=test_src, output=test_output)
        cloned = test_func["cloned"]

    generated = None
    if config_file and model:
        with open(config_file, "r") as f:
            config = yaml.safe_load(f)
        messages = [
            {"content": config["system"], "role": "system"},
            {
                "content": config["demonstration"].format(
                    **{"demo": random.choice(SWEBV_PS)}
                ),
                "role": "user",
            },
            {
                "content": config["instance"].format(**{"input": test_info}),
                "role": "user",
            },
        ]
        response = completion(model=model, messages=messages, n=1, temperature=0)
        generated = response.choices[0].message.content

    log_dir.mkdir(parents=True, exist_ok=True)
    with open(path_metadata, "w") as f:
        json.dump(
            {
                "test_idx": test_idx,
                "test_info": test_info,
                "test_output": test_output,
                "test_src": test_src,
                "generated": generated,
                "cost": completion_cost(response) if generated else 0,
            },
            f,
            indent=2,
        )
    with open(path_issue, "w") as f:
        f.write(generated if generated else test_info)
    rv = {"completed": 1, "timed_out": 0, "failed": 0}
    if cloned:
        rv["cleanup"] = instance["repo"].split("/")[-1]
    return rv


def main(dataset_path: str, config_file: str | None, model: str | None, n_workers: int):
    if config_file is not None:
        assert model is not None, "Model must be provided if config file is provided."
    if model is not None:
        assert config_file is not None, (
            "Config file must be provided if model is provided."
        )

    with open(dataset_path, "r") as f:
        dataset = json.load(f)
    print(f"Found {len(dataset)} task instances to generate instructions for")
    random.seed(24)

    stats = {"completed": 0, "timed_out": 0, "failed": 0}
    repos_to_remove = set()

    try:
        with ThreadPoolExecutor(max_workers=n_workers) as executor:
            futures = [
                executor.submit(_process_instance, instance, config_file, model)
                for instance in dataset
            ]

            with logging_redirect_tqdm():
                with tqdm(total=len(dataset), desc="Instances") as pbar:
                    for future in as_completed(futures):
                        result = future.result()
                        for k in stats.keys():
                            stats[k] += result[k]
                        if "cleanup" in result:
                            repos_to_remove.add(result["cleanup"])
                        pbar.set_postfix(stats, refresh=True)
                        pbar.update(1)
    except KeyboardInterrupt:
        print(f"Interrupted. Removing {len(repos_to_remove)} repos")
        if len(repos_to_remove) > 0:
            for repo in repos_to_remove:
                subprocess.run(f"rm -rf {repo}", shell=True)
        return

    # Attach issues to dataset
    for instance in dataset:
        log_dir = (
            LOG_DIR_ISSUE_GEN
            / "from_tests"
            / instance["repo"].split("/")[-1]
            / instance[KEY_INSTANCE_ID]
        )
        path_issue = log_dir / LOG_FILE_ISSUE
        if path_issue.exists():
            instance["problem_statement"] = path_issue.read_text()

    suffix = "ig_tests" if config_file is None else Path(config_file).stem
    with open(
        Path(dataset_path).parent / f"{Path(dataset_path).stem}__{suffix}.json", "w"
    ) as f:
        json.dump(dataset, f, indent=2)
        print(f"Wrote dataset with test code + traces to {f.name}")

    if len(repos_to_remove) > 0:
        print(f"Removing {len(repos_to_remove)} repos")
        for repo in repos_to_remove:
            subprocess.run(f"rm -rf {repo}", shell=True)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("dataset_path", type=str, help="Dataset files to process")
    parser.add_argument(
        "--config_file", type=str, help="Path to the configuration file."
    )
    parser.add_argument("--model", type=str, help="Model to use for rewriting.")
    parser.add_argument(
        "-n", "--n_workers", type=int, default=1, help="Number of workers to use"
    )
    args = parser.parse_args()
    main(**vars(args))



# Note: The code in this folder is heavily imported from SWE-bench's [collection code](https://github.com/SWE-bench/SWE-bench/tree/main/swebench/collect).
#!/usr/bin/env python3

"""Script to collect pull requests and convert them to candidate task instances"""

import argparse
import os
import traceback

from dotenv import load_dotenv
from multiprocessing import Pool
from pathlib import Path
from swesmith.bug_gen.mirror.collect.build_dataset import main as build_dataset
from swesmith.bug_gen.mirror.collect.print_pulls import main as print_pulls


load_dotenv()


def split_instances(input_list: list, n: int) -> list:
    """
    Split a list into n approximately equal length sublists

    Args:
        input_list (list): List to split
        n (int): Number of sublists to split into
    Returns:
        result (list): List of sublists
    """
    avg_length = len(input_list) // n
    remainder = len(input_list) % n
    result, start = [], 0

    for i in range(n):
        length = avg_length + 1 if i < remainder else avg_length
        sublist = input_list[start : start + length]
        result.append(sublist)
        start += length

    return result


def construct_data_files(data: dict):
    """
    Logic for combining multiple .all PR files into a single fine tuning dataset

    Args:
        data (dict): Dictionary containing the following keys:
            repos (list): List of repositories to retrieve instruction data for
            path_prs (str): Path to save PR data files to
            path_tasks (str): Path to save task instance data files to
            token (str): GitHub token to use for API requests
    """
    repos, path_prs, path_tasks, max_pulls, cutoff_date, token = (
        data["repos"],
        data["path_prs"],
        data["path_tasks"],
        data["max_pulls"],
        data["cutoff_date"],
        data["token"],
    )
    for repo in repos:
        repo = repo.strip(",").strip()
        repo_name = repo.split("/")[1]
        try:
            path_pr = os.path.join(path_prs, f"{repo_name}-prs.jsonl")
            if cutoff_date:
                path_pr = path_pr.replace(".jsonl", f"-{cutoff_date}.jsonl")
            if not os.path.exists(path_pr):
                print(f"Pull request data for {repo} not found, creating...")
                print_pulls(
                    repo, path_pr, token, max_pulls=max_pulls, cutoff_date=cutoff_date
                )
                print(f"✅ Successfully saved PR data for {repo} to {path_pr}")
            else:
                print(
                    f"📁 Pull request data for {repo} already exists at {path_pr}, skipping..."
                )

            path_task = os.path.join(path_tasks, f"{repo_name}-insts.jsonl")
            if not os.path.exists(path_task):
                print(f"Task instance data for {repo} not found, creating...")
                build_dataset(path_pr, path_task, token)
                print(
                    f"✅ Successfully saved task instance data for {repo} to {path_task}"
                )
            else:
                print(
                    f"📁 Task instance data for {repo} already exists at {path_task}, skipping..."
                )
        except Exception as e:
            print("-" * 80)
            print(f"Something went wrong for {repo}, skipping: {e}")
            print("Here is the full traceback:")
            traceback.print_exc()
            print("-" * 80)


def main(
    repos: list,
    path_prs: str,
    path_tasks: str,
    max_pulls: int = None,
    cutoff_date: str = None,
):
    """
    Spawns multiple threads given multiple GitHub tokens for collecting fine tuning data

    Args:
        repos (list): List of repositories to retrieve instruction data for
        path_prs (str): Path to save PR data files to
        path_tasks (str): Path to save task instance data files to
        cutoff_date (str): Cutoff date for PRs to consider in format YYYYMMDD
    """
    path_prs, path_tasks = os.path.abspath(path_prs), os.path.abspath(path_tasks)
    Path(path_prs).mkdir(exist_ok=True, parents=True)
    Path(path_tasks).mkdir(exist_ok=True, parents=True)
    print(f"Will save PR data to {path_prs}")
    print(f"Will save task instance data to {path_tasks}")
    print(f"Received following repos to create task instances for: {repos}")

    tokens = os.getenv("GITHUB_TOKENS")
    if not tokens:
        raise Exception(
            "Missing GITHUB_TOKENS, consider rerunning with GITHUB_TOKENS=$(gh auth token)"
        )
    tokens = tokens.split(",")
    data_task_lists = split_instances(repos, len(tokens))

    data_pooled = [
        {
            "repos": repos,
            "path_prs": path_prs,
            "path_tasks": path_tasks,
            "max_pulls": max_pulls,
            "cutoff_date": cutoff_date,
            "token": token,
        }
        for repos, token in zip(data_task_lists, tokens)
    ]

    with Pool(len(tokens)) as p:
        p.map(construct_data_files, data_pooled)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "--repos",
        nargs="+",
        help="List of repositories (e.g., `sqlfluff/sqlfluff`) to create task instances for",
    )
    parser.add_argument(
        "--path_prs", type=str, help="Path to folder to save PR data files to"
    )
    parser.add_argument(
        "--path_tasks",
        type=str,
        help="Path to folder to save task instance data files to",
    )
    parser.add_argument(
        "--max_pulls", type=int, help="Maximum number of pulls to log", default=None
    )
    parser.add_argument(
        "--cutoff_date",
        type=str,
        help="Cutoff date for PRs to consider in format YYYYMMDD",
        default=None,
    )
    args = parser.parse_args()
    main(**vars(args))
